@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,6 +31,8 @@
 #include "lib.h"
 #include <d3d9.h>
 
+Direct3D9Data *d3dd;
+
 typedef IDirect3D9*(WINAPI *pDirect3DCreate9)(UINT SDKVersion) ;
 typedef HRESULT(WINAPI *pDirect3DCreate9Ex)(UINT SDKVersion, IDirect3D9Ex **ppD3D) ;
 
@@ -49,9 +51,10 @@ class DevState {
 		LONG initRefCount;
 		LONG refCount;
 		LONG myRefCount;
-		bool bMyRefs;
+		DWORD dwMyThread;
 
 		LPDIRECT3DTEXTURE9 tex[NUM_TEXTS];
+		unsigned int uiCounter[NUM_TEXTS];
 
 		DevState();
 
@@ -70,7 +73,7 @@ static bool bPresenting = false;
 DevState::DevState() {
 	dev = NULL;
 	pSB = NULL;
-	bMyRefs = false;
+	dwMyThread = 0;
 	refCount = 0;
 	myRefCount = 0;
 	for (int i = 0;i < NUM_TEXTS;i++)
@@ -128,7 +131,7 @@ void DevState::draw() {
 			if (sm->texts[i].width == 0) {
 				y += iHeight / 4;
 			} else if (sm->texts[i].width > 0) {
-				if (!tex[i] || sm->texts[i].bUpdated) {
+				if (!tex[i] || (sm->texts[i].uiCounter != uiCounter[i])) {
 					if (tex[i])
 						tex[i]->Release();
 
@@ -143,7 +146,7 @@ void DevState::draw() {
 					}
 
 					tex[i]->UnlockRect(0);
-					sm->texts[i].bUpdated = false;
+					uiCounter[i] = sm->texts[i].uiCounter;
 				}
 				unsigned int w = lround(sm->texts[i].width * s);
 				texs.push_back(tex[i]);
@@ -215,8 +218,11 @@ void DevState::draw() {
 }
 
 void DevState::createCleanState() {
-	bool b = bMyRefs;
-	bMyRefs = true;
+	DWORD dwOldThread = dwMyThread;
+	if (dwOldThread) {
+		ods("CreateCleanState from other thread.");
+	}
+	dwMyThread = GetCurrentThreadId();
 
 	if (pSB)
 		pSB->Release();
@@ -263,7 +269,7 @@ void DevState::createCleanState() {
 	pStateBlock->Apply();
 	pStateBlock->Release();
 
-	bMyRefs = b;
+	dwMyThread = dwOldThread;
 }
 
 static HardHook hhCreateDevice;
@@ -277,7 +283,13 @@ static HardHook hhSwapPresent;
 
 static void doPresent(IDirect3DDevice9 *idd) {
 	DevState *ds = devMap[idd];
+
 	if (ds && sm->bShow) {
+		DWORD dwOldThread = ds->dwMyThread;
+		if (dwOldThread)
+			ods("doPresent from other thread");
+		ds->dwMyThread = GetCurrentThreadId();
+
 		IDirect3DSurface9 *pTarget = NULL;
 		IDirect3DSurface9 *pRenderTarget = NULL;
 		idd->GetBackBuffer(0, 0, D3DBACKBUFFER_TYPE_MONO, &pTarget);
@@ -285,8 +297,6 @@ static void doPresent(IDirect3DDevice9 *idd) {
 
 		ods("D3D9: doPresent Back %p RenderT %p",pTarget,pRenderTarget);
 
-		bool b = ds->bMyRefs;
-		ds->bMyRefs = true;
 
 		IDirect3DStateBlock9* pStateBlock = NULL;
 		idd->CreateStateBlock(D3DSBT_ALL, &pStateBlock);
@@ -309,14 +319,16 @@ static void doPresent(IDirect3DDevice9 *idd) {
 		pStateBlock->Apply();
 		pStateBlock->Release();
 
-		ds->bMyRefs = b;
-
 		pRenderTarget->Release();
 		pTarget->Release();
-		bVideoHooked = true;
+
+		ods("Finished ref is %d %d", ds->myRefCount, ds->refCount);
+		ds->dwMyThread = dwOldThread;
 	}
 }
 
+
+typedef HRESULT(__stdcall *SwapPresentType)(IDirect3DSwapChain9 *, CONST RECT *, CONST RECT *, HWND, CONST RGNDATA *, DWORD);
 static HRESULT __stdcall mySwapPresent(IDirect3DSwapChain9 * ids, CONST RECT *pSourceRect, CONST RECT *pDestRect, HWND hDestWindowOverride, CONST RGNDATA *pDirtyRegion, DWORD dwFlags) {
 	ods("D3D9: SwapChain Present");
 
@@ -331,61 +343,83 @@ static HRESULT __stdcall mySwapPresent(IDirect3DSwapChain9 * ids, CONST RECT *pS
 		}
 	}
 
+	SwapPresentType oSwapPresent;
+	oSwapPresent = (SwapPresentType) hhSwapPresent.call;
+
 	hhSwapPresent.restore();
-	HRESULT hr = ids->Present(pSourceRect,pDestRect,hDestWindowOverride,pDirtyRegion,dwFlags);
+	HRESULT hr = oSwapPresent(ids, pSourceRect,pDestRect,hDestWindowOverride,pDirtyRegion,dwFlags);
 	hhSwapPresent.inject();
 	return hr;
 }
 
+typedef HRESULT(__stdcall *PresentType)(IDirect3DDevice9 *, CONST RECT *, CONST RECT *, HWND, CONST RGNDATA *);
 static HRESULT __stdcall myPresent(IDirect3DDevice9 * idd, CONST RECT* pSourceRect,CONST RECT* pDestRect,HWND hDestWindowOverride,CONST RGNDATA* pDirtyRegion) {
 	ods("D3D9: Device Present");
 
 	doPresent(idd);
 
+	PresentType oPresent = (PresentType) hhPresent.call;
+
 	hhPresent.restore();
-	HRESULT hr = idd->Present(pSourceRect,pDestRect,hDestWindowOverride,pDirtyRegion);
+	HRESULT hr = oPresent(idd,pSourceRect,pDestRect,hDestWindowOverride,pDirtyRegion);
 	hhPresent.inject();
 	return hr;
 }
 
+typedef HRESULT(__stdcall *ResetType)(IDirect3DDevice9 *, D3DPRESENT_PARAMETERS *);
 static HRESULT __stdcall myReset(IDirect3DDevice9 * idd, D3DPRESENT_PARAMETERS *param) {
 	ods("D3D9: Chaining Reset");
 
 	DevState *ds = devMap[idd];
 	if (ds) {
-		bool b = ds->bMyRefs;
-		ds->bMyRefs = true;
+		DWORD dwOldThread = ds->dwMyThread;
+		if (dwOldThread)
+			ods("myReset from other thread");
+		ds->dwMyThread = GetCurrentThreadId();
+
 		ds->releaseAll();
-		ds->bMyRefs = b;
+		ds->dwMyThread = dwOldThread;
 	}
+
+
+	ResetType oReset = (ResetType) hhReset.call;
+
 	hhReset.restore();
-	HRESULT hr=idd->Reset(param);
+	HRESULT hr=oReset(idd, param);
 	hhReset.inject();
 
 	ds->createCleanState();
 	return hr;
 }
 
+typedef ULONG(__stdcall *AddRefType)(IDirect3DDevice9 *);
+
 static ULONG __stdcall myAddRef(IDirect3DDevice9 *idd) {
+	Mutex m;
 	DevState *ds = devMap[idd];
 	if (ds) {
-		if (ds->bMyRefs)
+		if (ds->dwMyThread == GetCurrentThreadId()) {
 			ds->myRefCount++;
-		else
+		} else
 			ds->refCount++;
 		return ds->refCount + ds->initRefCount;
 	}
-	ods("D3D9: Unknown device, chaining AddRef");
+	AddRefType oAddRef = (AddRefType) hhAddRef.call;
+
 	hhAddRef.restore();
-	LONG res = idd->AddRef();
+	LONG res = oAddRef(idd);
 	hhAddRef.inject();
+	ods("D3D9: Chaining AddRef: %d", res);
 	return res;
 }
 
+typedef ULONG(__stdcall *ReleaseType)(IDirect3DDevice9 *);
 static ULONG __stdcall myRelease(IDirect3DDevice9 *idd) {
+	Mutex m;
 	DevState *ds = devMap[idd];
+
 	if (ds) {
-		if (ds->bMyRefs) {
+		if (ds->dwMyThread == GetCurrentThreadId()) {
 			ds->myRefCount--;
 			return ds->refCount + ds->initRefCount;
 		} else {
@@ -395,38 +429,70 @@ static ULONG __stdcall myRelease(IDirect3DDevice9 *idd) {
 		if (ds->refCount >= 0)
 			return ds->refCount + ds->initRefCount;
 
-		ods("D3D9: Final release");
+		ods("D3D9: Final release. MyRefs = %d, Tot = %d", ds->myRefCount, ds->refCount);
+
+		DWORD dwOldThread = ds->dwMyThread;
+		if (dwOldThread)
+			ods("finalRelease from other thread");
+		ds->dwMyThread = GetCurrentThreadId();
 
-		bool b = ds->bMyRefs;
-		ds->bMyRefs = true;
 		ds->releaseAll();
-		ds->bMyRefs = b;
+
+		ds->dwMyThread = dwOldThread;
+
+		ods("D3D9: Final release, MyRefs = %d Tot = %d", ds->myRefCount, ds->refCount);
 
 		devMap.erase(idd);
 		delete ds;
 	}
-	ods("D3D9: Chaining Release");
+	ReleaseType oRelease = (ReleaseType) hhRelease.call;
 	hhRelease.restore();
-	LONG res = idd->Release();
+	LONG res = oRelease(idd);
 	hhRelease.inject();
+	ods("D3D9: Chaining Release: %d", res);
 	return res;
 }
 
-
-
+typedef HRESULT(__stdcall *CreateDeviceType)(IDirect3D9 *, UINT, D3DDEVTYPE, HWND, DWORD, D3DPRESENT_PARAMETERS *, IDirect3DDevice9 **);
 static HRESULT __stdcall myCreateDevice(IDirect3D9 * id3d, UINT Adapter, D3DDEVTYPE DeviceType, HWND hFocusWindow, DWORD BehaviorFlags, D3DPRESENT_PARAMETERS *pPresentationParameters, IDirect3DDevice9 **ppReturnedDeviceInterface) {
 	ods("D3D9: Chaining CreateDevice");
+	Mutex m;
 
 //	BehaviorFlags &= ~D3DCREATE_PUREDEVICE;
 
+	CreateDeviceType oCreateDevice = (CreateDeviceType) hhCreateDevice.call;
+
 	hhCreateDevice.restore();
-	HRESULT hr=id3d->CreateDevice(Adapter,DeviceType,hFocusWindow,BehaviorFlags,pPresentationParameters,ppReturnedDeviceInterface);
+	HRESULT hr=oCreateDevice(id3d, Adapter,DeviceType,hFocusWindow,BehaviorFlags,pPresentationParameters,ppReturnedDeviceInterface);
 	hhCreateDevice.inject();
 
 	if (FAILED(hr))
 		return hr;
 
 	IDirect3DDevice9 *idd = *ppReturnedDeviceInterface;
+	IDirect3DSwapChain9 *pSwap = NULL;
+
+	// Get real interface, please.
+	bool bfound;
+	do {
+		bfound = false;
+		idd->GetSwapChain(0, &pSwap);
+		if (pSwap) {
+			IDirect3DDevice9 *idorig = NULL;
+			if (SUCCEEDED(pSwap->GetDevice(&idorig))) {
+				if (idorig != idd) {
+					ods("Prepatched device, using original. %p => %p", idorig, idd);
+					if (idd != *ppReturnedDeviceInterface)
+						idd->Release();
+					idd = idorig;
+					bfound = true;
+				} else {
+					idorig->Release();
+				}
+			}
+			pSwap->Release();
+		}
+	} while (bfound);
 
 	DevState *ds = new DevState;
 	ds->dev = idd;
@@ -437,22 +503,14 @@ static HRESULT __stdcall myCreateDevice(IDirect3D9 * id3d, UINT Adapter, D3DDEVT
 	devMap[idd] = ds;
 
 	hhAddRef.setupInterface(idd, 1, reinterpret_cast<voidFunc>(myAddRef));
-	hhAddRef.inject();
-
 	hhRelease.setupInterface(idd, 2, reinterpret_cast<voidFunc>(myRelease));
-	hhRelease.inject();
-
 	hhReset.setupInterface(idd, 16, reinterpret_cast<voidFunc>(myReset));
-	hhReset.inject();
-
 	hhPresent.setupInterface(idd, 17, reinterpret_cast<voidFunc>(myPresent));
-	hhPresent.inject();
 
-	IDirect3DSwapChain9 *pSwap = NULL;
+	pSwap = NULL;
 	idd->GetSwapChain(0, &pSwap);
 	if (pSwap) {
 		hhSwapPresent.setupInterface(pSwap, 3, reinterpret_cast<voidFunc>(mySwapPresent));
-		hhSwapPresent.inject();
 		pSwap->Release();
 	} else {
 		ods("D3D9: Failed to get swapchain");
@@ -462,13 +520,17 @@ static HRESULT __stdcall myCreateDevice(IDirect3D9 * id3d, UINT Adapter, D3DDEVT
 	return hr;
 }
 
+typedef HRESULT(__stdcall *CreateDeviceExType)(IDirect3D9Ex *, UINT, D3DDEVTYPE, HWND, DWORD, D3DPRESENT_PARAMETERS *, D3DDISPLAYMODEEX *, IDirect3DDevice9Ex **);
 static HRESULT __stdcall myCreateDeviceEx(IDirect3D9Ex * id3d, UINT Adapter, D3DDEVTYPE DeviceType, HWND hFocusWindow, DWORD BehaviorFlags, D3DPRESENT_PARAMETERS *pPresentationParameters, D3DDISPLAYMODEEX* pFullscreenDisplayMode, IDirect3DDevice9Ex** ppReturnedDeviceInterface) {
+	Mutex m;
 	ods("D3D9: Chaining CreateDeviceEx");
 
 //	BehaviorFlags &= ~D3DCREATE_PUREDEVICE;
 
+	CreateDeviceExType oCreateDeviceEx = (CreateDeviceExType) hhCreateDeviceEx.call;
+
 	hhCreateDeviceEx.restore();
-	HRESULT hr=id3d->CreateDeviceEx(Adapter,DeviceType,hFocusWindow,BehaviorFlags,pPresentationParameters,pFullscreenDisplayMode,ppReturnedDeviceInterface);
+	HRESULT hr=oCreateDeviceEx(id3d, Adapter,DeviceType,hFocusWindow,BehaviorFlags,pPresentationParameters,pFullscreenDisplayMode,ppReturnedDeviceInterface);
 	hhCreateDeviceEx.inject();
 
 	if (FAILED(hr))
@@ -485,22 +547,14 @@ static HRESULT __stdcall myCreateDeviceEx(IDirect3D9Ex * id3d, UINT Adapter, D3D
 	devMap[idd] = ds;
 
 	hhAddRef.setupInterface(idd, 1, reinterpret_cast<voidFunc>(myAddRef));
-	hhAddRef.inject();
-
 	hhRelease.setupInterface(idd, 2, reinterpret_cast<voidFunc>(myRelease));
-	hhRelease.inject();
-
 	hhReset.setupInterface(idd, 16, reinterpret_cast<voidFunc>(myReset));
-	hhReset.inject();
-
 	hhPresent.setupInterface(idd, 17, reinterpret_cast<voidFunc>(myPresent));
-	hhPresent.inject();
 
 	IDirect3DSwapChain9 *pSwap = NULL;
 	idd->GetSwapChain(0, &pSwap);
 	if (pSwap) {
 		hhSwapPresent.setupInterface(pSwap, 3, reinterpret_cast<voidFunc>(mySwapPresent));
-		hhSwapPresent.inject();
 		pSwap->Release();
 	} else {
 		ods("D3D9: Failed to get swapchain for DevEx");
@@ -510,31 +564,25 @@ static HRESULT __stdcall myCreateDeviceEx(IDirect3D9Ex * id3d, UINT Adapter, D3D
 	return hr;
 }
 
+static void HookCreateRaw(voidFunc vfCreate) {
+	ods("D3D9: Injecting CreateDevice Raw");
+	hhCreateDevice.setup(vfCreate, reinterpret_cast<voidFunc>(myCreateDevice));
+}
+
 static void HookCreate(IDirect3D9 *pD3D) {
 	ods("D3D9: Injecting CreateDevice");
 
-	// Add a ref to ourselves; we do NOT want to get unloaded directly from this process.
-	GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, reinterpret_cast<char *>(&HookCreate), &hSelf);
-
 	hhCreateDevice.setupInterface(pD3D, 16, reinterpret_cast<voidFunc>(myCreateDevice));
-	hhCreateDevice.inject();
 }
 
 static void HookCreateEx(IDirect3D9Ex *pD3D) {
 	ods("D3D9Ex: Injecting CreateDevice / CreateDeviceEx");
 
-	// Add a ref to ourselves; we do NOT want to get unloaded directly from this process.
-	GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, reinterpret_cast<char *>(&HookCreate), &hSelf);
-
 	hhCreateDevice.setupInterface(pD3D, 16, reinterpret_cast<voidFunc>(myCreateDevice));
-	hhCreateDevice.inject();
-
 	hhCreateDeviceEx.setupInterface(pD3D, 20, reinterpret_cast<voidFunc>(myCreateDeviceEx));
-	hhCreateDeviceEx.inject();
 }
 
-
-void checkD3D9Hook() {
+void checkD3D9Hook(bool preonly) {
 	if (bChaining) {
 		return;
 		ods("D3D9: Causing a chain");
@@ -546,31 +594,87 @@ void checkD3D9Hook() {
 
 	if (hD3D != NULL) {
 		if (! bHooked) {
-			char procname[1024];
-			GetModuleFileName(NULL, procname, 1024);
-			ods("D3D9: CreateWnd in unhooked D3D App %s", procname);
+			char procname[2048];
+			GetModuleFileName(NULL, procname, 2048);
+			fods("D3D9: CreateWnd in unhooked D3D App %s", procname);
 			bHooked = true;
 
-			pDirect3DCreate9 d3dc9 = reinterpret_cast<pDirect3DCreate9>(GetProcAddress(hD3D, "Direct3DCreate9"));
-			if (d3dc9) {
-				IDirect3D9 *id3d9 = d3dc9(D3D_SDK_VERSION);
-				if (id3d9) {
-					HookCreate(id3d9);
-					IDirect3D9_Release(id3d9);
-				}
-			}
+			// Add a ref to ourselves; we do NOT want to get unloaded directly from this process.
+			GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, reinterpret_cast<char *>(&HookCreate), &hSelf);
 
-			pDirect3DCreate9Ex d3dc9ex = reinterpret_cast<pDirect3DCreate9Ex>(GetProcAddress(hD3D, "Direct3DCreate9Ex"));
-			if (d3dc9ex) {
-				IDirect3D9Ex *id3d9ex = NULL;
-				d3dc9ex(D3D_SDK_VERSION, &id3d9ex);
-				if (id3d9ex) {
-					HookCreateEx(id3d9ex);
-					IDirect3D9Ex_Release(id3d9ex);
+			// Can we use the prepatch data?
+			GetModuleFileName(hD3D, procname, 2048);
+			if (_stricmp(d3dd->cFileName, procname) == 0) {
+				unsigned char *raw = (unsigned char *) hD3D;
+				HookCreateRaw((voidFunc)(raw + d3dd->iOffsetCreate));
+			} else if (! preonly) {
+				fods("D3D9 Interface changed, can't rawpatch");
+				pDirect3DCreate9 d3dc9 = reinterpret_cast<pDirect3DCreate9>(GetProcAddress(hD3D, "Direct3DCreate9"));
+				ods("Got %p", d3dc9);
+				if (d3dc9) {
+					IDirect3D9 *id3d9 = d3dc9(D3D_SDK_VERSION);
+					if (id3d9) {
+						HookCreate(id3d9);
+						id3d9->Release();
+					} else {
+						ods("Failed Direct3DCreate9");
+					}
+				} else {
+					ods("D3D Library without Direct3DCreate9?");
 				}
+			} else {
+				bHooked = false;
 			}
 		}
 	}
 
 	bChaining = false;
 }
+
+extern "C" __declspec(dllexport) void __cdecl PrepareD3D9() {
+	ods("Preparing static data for D3D9 Injection");
+
+	char buffb[2048];
+
+	HMODULE hD3D = LoadLibrary("D3D9.DLL");
+	HMODULE hRef;
+
+	if (hD3D != NULL) {
+		GetModuleFileName(hD3D, d3dd->cFileName, 2048);
+		pDirect3DCreate9 d3dc9 = reinterpret_cast<pDirect3DCreate9>(GetProcAddress(hD3D, "Direct3DCreate9"));
+		if (! d3dc9) {
+			ods("D3D9 Library without Direct3DCreate9");
+		} else {
+			if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (const char *) d3dc9, &hRef)) {
+				ods("Failed to get module for D3D9");
+			} else {
+				GetModuleFileName(hRef, buffb, 2048);
+				if (_stricmp(d3dd->cFileName, buffb) != 0) {
+					ods("Direct3DCreate9 is not in D3D9 library");
+				} else {
+					IDirect3D9 *id3d9 = d3dc9(D3D_SDK_VERSION);
+					if (id3d9) {
+						void ***vtbl = (void ***) id3d9;
+						void *pCreate = (*vtbl)[16];
+
+						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pCreate, &hRef)) {
+							ods("Failed to get module for CreateDevice");
+						} else {
+							GetModuleFileName(hRef, buffb, 2048);
+							if (_stricmp(d3dd->cFileName, buffb) != 0) {
+								ods("CreateDevice is not in D3D9 library");
+							} else {
+								unsigned char *b = (unsigned char *) pCreate;
+								unsigned char *a = (unsigned char *) hD3D;
+								d3dd->iOffsetCreate = b-a;
+								ods("Successfully found prepatch offset: %p %p %p: %d", hD3D, d3dc9, pCreate, d3dd->iOffsetCreate);
+							}
+						}
+						id3d9->Release();
+					}
+				}
+			}
+		}
+		FreeLibrary(hD3D);
+	}
+}
