@@ -54,10 +54,6 @@
 #include "vorbis_psy.h"
 #endif
 
-#ifndef M_PI
-#define M_PI           3.14159265358979323846  /* pi */
-#endif
-
 #ifndef NULL
 #define NULL 0
 #endif
@@ -76,11 +72,11 @@
 
 
 #ifdef FIXED_POINT
-const spx_word32_t ol_gain_table[32]={18900, 25150, 33468, 44536, 59265, 78865, 104946, 139653, 185838, 247297, 329081, 437913, 582736, 775454, 1031906, 1373169, 1827293, 2431601, 3235761, 4305867, 5729870, 7624808, 10146425, 13501971, 17967238, 23909222, 31816294, 42338330, 56340132, 74972501, 99766822, 132760927};
-const spx_word16_t exc_gain_quant_scal3_bound[7]={1841, 3883, 6051, 8062, 10444, 13580, 18560};
-const spx_word16_t exc_gain_quant_scal3[8]={1002, 2680, 5086, 7016, 9108, 11781, 15380, 21740};
-const spx_word16_t exc_gain_quant_scal1_bound[1]={14385};
-const spx_word16_t exc_gain_quant_scal1[2]={11546, 17224};
+static const spx_word32_t ol_gain_table[32]={18900, 25150, 33468, 44536, 59265, 78865, 104946, 139653, 185838, 247297, 329081, 437913, 582736, 775454, 1031906, 1373169, 1827293, 2431601, 3235761, 4305867, 5729870, 7624808, 10146425, 13501971, 17967238, 23909222, 31816294, 42338330, 56340132, 74972501, 99766822, 132760927};
+static const spx_word16_t exc_gain_quant_scal3_bound[7]={1841, 3883, 6051, 8062, 10444, 13580, 18560};
+static const spx_word16_t exc_gain_quant_scal3[8]={1002, 2680, 5086, 7016, 9108, 11781, 15380, 21740};
+static const spx_word16_t exc_gain_quant_scal1_bound[1]={14385};
+static const spx_word16_t exc_gain_quant_scal1[2]={11546, 17224};
 
 #define LSP_MARGIN 16
 #define LSP_DELTA1 6553
@@ -88,10 +84,10 @@ const spx_word16_t exc_gain_quant_scal1[2]={11546, 17224};
 
 #else
 
-const float exc_gain_quant_scal3_bound[7]={0.112338f, 0.236980f, 0.369316f, 0.492054f, 0.637471f, 0.828874f, 1.132784f};
-const float exc_gain_quant_scal3[8]={0.061130f, 0.163546f, 0.310413f, 0.428220f, 0.555887f, 0.719055f, 0.938694f, 1.326874f};
-const float exc_gain_quant_scal1_bound[1]={0.87798f};
-const float exc_gain_quant_scal1[2]={0.70469f, 1.05127f};
+static const float exc_gain_quant_scal3_bound[7]={0.112338f, 0.236980f, 0.369316f, 0.492054f, 0.637471f, 0.828874f, 1.132784f};
+static const float exc_gain_quant_scal3[8]={0.061130f, 0.163546f, 0.310413f, 0.428220f, 0.555887f, 0.719055f, 0.938694f, 1.326874f};
+static const float exc_gain_quant_scal1_bound[1]={0.87798f};
+static const float exc_gain_quant_scal1[2]={0.70469f, 1.05127f};
 
 #define LSP_MARGIN .002f
 #define LSP_DELTA1 .2f
@@ -419,7 +415,6 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
    int ol_pitch;
    spx_word16_t ol_pitch_coef;
    spx_word32_t ol_gain;
-   VARDECL(spx_word16_t *ringing);
    VARDECL(spx_word16_t *target);
    VARDECL(spx_sig_t *innov);
    VARDECL(spx_word32_t *exc32);
@@ -435,7 +430,6 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
    VARDECL(spx_coef_t *interp_qlpc);
    char *stack;
    VARDECL(spx_word16_t *syn_resp);
-   VARDECL(spx_word16_t *real_exc);
    
    spx_word32_t ener=0;
    spx_word16_t fine_gain;
@@ -470,16 +464,17 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       ALLOC(autocorr, NB_ORDER+1, spx_word16_t);
       /* Window for analysis */
       for (i=0;i<NB_WINDOW_SIZE-NB_FRAME_SIZE;i++)
-         w_sig[i] = EXTRACT16(SHR32(MULT16_16(st->winBuf[i],st->window[i]),SIG_SHIFT));
+         w_sig[i] = MULT16_16_Q15(st->winBuf[i],st->window[i]);
       for (;i<NB_WINDOW_SIZE;i++)
-         w_sig[i] = EXTRACT16(SHR32(MULT16_16(in[i-NB_WINDOW_SIZE+NB_FRAME_SIZE],st->window[i]),SIG_SHIFT));
+         w_sig[i] = MULT16_16_Q15(in[i-NB_WINDOW_SIZE+NB_FRAME_SIZE],st->window[i]);
       /* Compute auto-correlation */
       _spx_autocorr(w_sig, autocorr, NB_ORDER+1, NB_WINDOW_SIZE);
       autocorr[0] = ADD16(autocorr[0],MULT16_16_Q15(autocorr[0],st->lpc_floor)); /* Noise floor in auto-correlation domain */
 
       /* Lag windowing: equivalent to filtering in the power-spectrum domain */
       for (i=0;i<NB_ORDER+1;i++)
-         autocorr[i] = MULT16_16_Q14(autocorr[i],st->lagWindow[i]);
+         autocorr[i] = MULT16_16_Q15(autocorr[i],st->lagWindow[i]);
+      autocorr[0] = ADD16(autocorr[0],1);
 
       /* Levinson-Durbin */
       _spx_lpc(lpc, autocorr, NB_ORDER);
@@ -506,9 +501,7 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
          for (i=0;i<NB_ORDER;i++)
             interp_lsp[i] = lsp[i];
       else
-         lsp_interpolate(st->old_lsp, lsp, interp_lsp, NB_ORDER, NB_NB_SUBFRAMES, NB_NB_SUBFRAMES<<1);
-
-      lsp_enforce_margin(interp_lsp, NB_ORDER, LSP_MARGIN);
+         lsp_interpolate(st->old_lsp, lsp, interp_lsp, NB_ORDER, NB_NB_SUBFRAMES, NB_NB_SUBFRAMES<<1, LSP_MARGIN);
 
       /* Compute interpolated LPCs (unquantized) for whole frame*/
       lsp_to_lpc(interp_lsp, interp_lpc, NB_ORDER,stack);
@@ -560,9 +553,10 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       }
       
       /*Compute "real" excitation*/
-      SPEEX_COPY(st->exc, st->winBuf, diff);
-      SPEEX_COPY(st->exc+diff, in, NB_FRAME_SIZE-diff);
-      fir_mem16(st->exc, interp_lpc, st->exc, NB_FRAME_SIZE, NB_ORDER, st->mem_exc, stack);
+      /*SPEEX_COPY(st->exc, st->winBuf, diff);
+      SPEEX_COPY(st->exc+diff, in, NB_FRAME_SIZE-diff);*/
+      fir_mem16(st->winBuf, interp_lpc, st->exc, diff, NB_ORDER, st->mem_exc, stack);
+      fir_mem16(in, interp_lpc, st->exc+diff, NB_FRAME_SIZE-diff, NB_ORDER, st->mem_exc, stack);
 
       /* Compute open-loop excitation gain */
       {
@@ -803,9 +797,7 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
    ALLOC(target, NB_SUBFRAME_SIZE, spx_word16_t);
    ALLOC(innov, NB_SUBFRAME_SIZE, spx_sig_t);
    ALLOC(exc32, NB_SUBFRAME_SIZE, spx_word32_t);
-   ALLOC(ringing, NB_SUBFRAME_SIZE, spx_word16_t);
    ALLOC(syn_resp, NB_SUBFRAME_SIZE, spx_word16_t);
-   ALLOC(real_exc, NB_SUBFRAME_SIZE, spx_word16_t);
    ALLOC(mem, NB_ORDER, spx_mem_t);
 
    /* Loop on sub-frames */
@@ -813,7 +805,7 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
    {
       int   offset;
       spx_word16_t *sw;
-      spx_word16_t *exc;
+      spx_word16_t *exc, *inBuf;
       int pitch;
       int response_bound = NB_SUBFRAME_SIZE;
 
@@ -825,12 +817,8 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       sw=st->sw+offset;
       
       /* LSP interpolation (quantized and unquantized) */
-      lsp_interpolate(st->old_lsp, lsp, interp_lsp, NB_ORDER, sub, NB_NB_SUBFRAMES);
-      lsp_interpolate(st->old_qlsp, qlsp, interp_qlsp, NB_ORDER, sub, NB_NB_SUBFRAMES);
-
-      /* Make sure the filters are stable */
-      lsp_enforce_margin(interp_lsp, NB_ORDER, LSP_MARGIN);
-      lsp_enforce_margin(interp_qlsp, NB_ORDER, LSP_MARGIN);
+      lsp_interpolate(st->old_lsp, lsp, interp_lsp, NB_ORDER, sub, NB_NB_SUBFRAMES, LSP_MARGIN);
+      lsp_interpolate(st->old_qlsp, qlsp, interp_qlsp, NB_ORDER, sub, NB_NB_SUBFRAMES, LSP_MARGIN);
 
       /* Compute interpolated LPCs (quantized and unquantized) */
       lsp_to_lpc(interp_lsp, interp_lpc, NB_ORDER,stack);
@@ -865,16 +853,12 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
 
       /*FIXME: This will break if we change the window size */
       speex_assert(NB_WINDOW_SIZE-NB_FRAME_SIZE == NB_SUBFRAME_SIZE);
-      {
-         spx_word16_t *buf;
-         if (sub==0)
-            buf = st->winBuf;
-         else
-            buf = &in[((sub-1)*NB_SUBFRAME_SIZE)];
-         for (i=0;i<NB_SUBFRAME_SIZE;i++)
-            real_exc[i] = sw[i] = buf[i];
-      }
-      fir_mem16(real_exc, interp_qlpc, real_exc, NB_SUBFRAME_SIZE, NB_ORDER, st->mem_exc2, stack);
+      if (sub==0)
+         inBuf = st->winBuf;
+      else
+         inBuf = &in[((sub-1)*NB_SUBFRAME_SIZE)];
+      for (i=0;i<NB_SUBFRAME_SIZE;i++)
+         sw[i] = inBuf[i];
       
       if (st->complexity==0)
          response_bound >>= 1;
@@ -886,18 +870,18 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       for (i=0;i<NB_ORDER;i++)
          mem[i]=SHL32(st->mem_sp[i],1);
       for (i=0;i<NB_SUBFRAME_SIZE;i++)
-         ringing[i] = VERY_SMALL;
+         exc[i] = VERY_SMALL;
 #ifdef SHORTCUTS2
-      iir_mem16(ringing, interp_qlpc, ringing, response_bound, NB_ORDER, mem, stack);
+      iir_mem16(exc, interp_qlpc, exc, response_bound, NB_ORDER, mem, stack);
       for (i=0;i<NB_ORDER;i++)
          mem[i]=SHL32(st->mem_sw[i],1);
-      filter10(ringing, st->bw_lpc1, st->bw_lpc2, ringing, response_bound, mem, stack);
-      SPEEX_MEMSET(&ringing[response_bound], 0, NB_SUBFRAME_SIZE-response_bound);
+      filter10(exc, st->bw_lpc1, st->bw_lpc2, exc, response_bound, mem, stack);
+      SPEEX_MEMSET(&exc[response_bound], 0, NB_SUBFRAME_SIZE-response_bound);
 #else
-      iir_mem16(ringing, interp_qlpc, ringing, NB_SUBFRAME_SIZE, NB_ORDER, mem, stack);
+      iir_mem16(exc, interp_qlpc, exc, NB_SUBFRAME_SIZE, NB_ORDER, mem, stack);
       for (i=0;i<NB_ORDER;i++)
          mem[i]=SHL32(st->mem_sw[i],1);
-      filter10(ringing, bw_lpc1, bw_lpc2, ringing, NB_SUBFRAME_SIZE, mem, stack);
+      filter10(exc, bw_lpc1, bw_lpc2, exc, NB_SUBFRAME_SIZE, mem, stack);
 #endif
       
       /* Compute weighted signal */
@@ -911,11 +895,11 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       
       /* Compute target signal (saturation prevents overflows on clipped input speech) */
       for (i=0;i<NB_SUBFRAME_SIZE;i++)
-         target[i]=EXTRACT16(SATURATE(SUB32(sw[i],PSHR32(ringing[i],1)),32767));
-
-      /* Reset excitation */
-      SPEEX_MEMSET(exc, 0, NB_SUBFRAME_SIZE);
+         target[i]=EXTRACT16(SATURATE(SUB32(sw[i],PSHR32(exc[i],1)),32767));
 
+      for (i=0;i<NB_SUBFRAME_SIZE;i++)
+         exc[i] = inBuf[i];
+      fir_mem16(exc, interp_qlpc, exc, NB_SUBFRAME_SIZE, NB_ORDER, st->mem_exc2, stack);
       /* If we have a long-term predictor (otherwise, something's wrong) */
       speex_assert (SUBMODE(ltp_quant));
       {
@@ -957,11 +941,11 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
       /* Quantization of innovation */
       SPEEX_MEMSET(innov, 0, NB_SUBFRAME_SIZE);
       
-      /* FIXME: Make sure this is save from overflows (so far so good) */
+      /* FIXME: Make sure this is safe from overflows (so far so good) */
       for (i=0;i<NB_SUBFRAME_SIZE;i++)
-         real_exc[i] = EXTRACT16(SUB32(EXTEND32(real_exc[i]), PSHR32(exc32[i],SIG_SHIFT-1)));
+         exc[i] = EXTRACT16(SUB32(EXTEND32(exc[i]), PSHR32(exc32[i],SIG_SHIFT-1)));
       
-      ener = SHL32(EXTEND32(compute_rms16(real_exc, NB_SUBFRAME_SIZE)),SIG_SHIFT);
+      ener = SHL32(EXTEND32(compute_rms16(exc, NB_SUBFRAME_SIZE)),SIG_SHIFT);
       
       /*FIXME: Should use DIV32_16 and make sure result fits in 16 bits */
 #ifdef FIXED_POINT
@@ -1009,9 +993,6 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
          /* De-normalize innovation and update excitation */
          signal_mul(innov, innov, ener, NB_SUBFRAME_SIZE);
          
-         for (i=0;i<NB_SUBFRAME_SIZE;i++)
-            exc[i] = EXTRACT16(SATURATE32(PSHR32(ADD32(SHL32(exc32[i],1),innov[i]),SIG_SHIFT),32767));
-
          /* In some (rare) modes, we do a second search (more bits) to reduce noise even more */
          if (SUBMODE(double_codebook)) {
             char *tmp_stack=stack;
@@ -1030,10 +1011,8 @@ int nb_encode(void *state, void *vin, SpeexBits *bits)
          }
          for (i=0;i<NB_SUBFRAME_SIZE;i++)
             exc[i] = EXTRACT16(SATURATE32(PSHR32(ADD32(SHL32(exc32[i],1),innov[i]),SIG_SHIFT),32767));
-#ifndef DISABLE_WIDEBAND
          if (st->innov_rms_save)
             st->innov_rms_save[sub] = compute_rms(innov, NB_SUBFRAME_SIZE);
-#endif
       }
 
       /* Final signal synthesis from excitation */
@@ -1786,10 +1765,7 @@ int nb_decode(void *state, SpeexBits *bits, void *vout)
       exc=st->exc+offset;
 
       /* LSP interpolation (quantized and unquantized) */
-      lsp_interpolate(st->old_qlsp, qlsp, interp_qlsp, NB_ORDER, sub, NB_NB_SUBFRAMES);
-
-      /* Make sure the LSP's are stable */
-      lsp_enforce_margin(interp_qlsp, NB_ORDER, LSP_MARGIN);
+      lsp_interpolate(st->old_qlsp, qlsp, interp_qlsp, NB_ORDER, sub, NB_NB_SUBFRAMES, LSP_MARGIN);
 
       /* Compute interpolated LPCs (unquantized) */
       lsp_to_lpc(interp_qlsp, ak, NB_ORDER, stack);
