@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -35,6 +35,8 @@
 #include "Timer.h"
 
 class CryptState {
+	private:
+		Q_DISABLE_COPY(CryptState)
 	public:
 		unsigned char raw_key[AES_BLOCK_SIZE];
 		unsigned char encrypt_iv[AES_BLOCK_SIZE];
