@@ -136,6 +136,9 @@ Message *Message::networkToMessage(PacketDataStream &qdsIn) {
 		case ChannelRename:
 			mMsg = new MessageChannelRename();
 			break;
+		case ChannelDescUpdate:
+			mMsg = new MessageChannelDescUpdate();
+			break;
 		case TextMessage:
 			mMsg = new MessageTextMessage();
 			break;
@@ -163,6 +166,12 @@ Message *Message::networkToMessage(PacketDataStream &qdsIn) {
 		case CryptSync:
 			mMsg = new MessageCryptSync();
 			break;
+		case ContextAction:
+			mMsg = new MessageContextAction();
+			break;
+		case ContextAddAction:
+			mMsg = new MessageContextAddAction();
+			break;
 		default:
 			qWarning("Message: Type %d (session %d, size %d) is unknown type", iMessageType, uiSession, qdsIn.capacity());
 	}
@@ -246,6 +255,9 @@ void MessageHandler::dispatch(Connection *cCon, Message *msg) {
 		case Message::ChannelRename:
 			msgChannelRename(cCon, static_cast<MessageChannelRename *>(msg));
 			break;
+		case Message::ChannelDescUpdate:
+			msgChannelDescUpdate(cCon, static_cast<MessageChannelDescUpdate *>(msg));
+			break;
 		case Message::TextMessage:
 			msgTextMessage(cCon, static_cast<MessageTextMessage *>(msg));
 			break;
@@ -273,6 +285,12 @@ void MessageHandler::dispatch(Connection *cCon, Message *msg) {
 		case Message::CryptSync:
 			msgCryptSync(cCon, static_cast<MessageCryptSync *>(msg));
 			break;
+		case Message::ContextAction:
+			msgContextAction(cCon, static_cast<MessageContextAction *>(msg));
+			break;
+		case Message::ContextAddAction:
+			msgContextAddAction(cCon, static_cast<MessageContextAddAction *>(msg));
+			break;
 		default:
 			qFatal("MessageHandler called with unknown message type %d", msg->messageType());
 	}
@@ -505,6 +523,14 @@ void MessageChannelRename::restoreStream(PacketDataStream &qdsIn) {
 	qdsIn >> iId >> qsName;
 }
 
+void MessageChannelDescUpdate::saveStream(PacketDataStream &qdsOut) const {
+	qdsOut << iId << qsDesc;
+}
+
+void MessageChannelDescUpdate::restoreStream(PacketDataStream &qdsIn) {
+	qdsIn >> iId >> qsDesc;
+}
+
 void MessageTextMessage::saveStream(PacketDataStream &qdsOut) const {
 	qdsOut << uiVictim;
 	qdsOut << iChannel;
@@ -600,6 +626,40 @@ bool MessageCryptSync::isValid() const {
 	return (qbaNonce.isEmpty() || (qbaNonce.size() == AES_BLOCK_SIZE));
 }
 
+void MessageContextAddAction::saveStream(PacketDataStream &qdsOut) const {
+	qdsOut << qsAction;
+	qdsOut << qsText;
+	qdsOut << static_cast<int>(ctx);
+}
+
+void MessageContextAddAction::restoreStream(PacketDataStream &qdsIn) {
+	qdsIn >> qsAction;
+	qdsIn >> qsText;
+	int c;
+	qdsIn >> c;
+	ctx = static_cast<Context>(c);
+}
+
+bool MessageContextAddAction::isValid() const {
+	return (! qsAction.isEmpty() && (ctx & (CtxServer | CtxPlayer | CtxChannel)));
+}
+
+void MessageContextAction::saveStream(PacketDataStream &qdsOut) const {
+	qdsOut << qsAction;
+	qdsOut << uiVictim;
+	qdsOut << iChannel;
+}
+
+void MessageContextAction::restoreStream(PacketDataStream &qdsIn) {
+	qdsIn >> qsAction;
+	qdsIn >> uiVictim;
+	qdsIn >> iChannel;
+}
+
+bool MessageContextAction::isValid() const {
+	return (! qsAction.isEmpty());
+}
+
 PacketDataStream & operator<< (PacketDataStream & out, const MessageEditACL::GroupStruct &gs) {
 	out << gs.qsName;
 	out << gs.bInherited;
