@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -40,7 +40,9 @@ class ALSAAudioOutput;
 class ALSAAudioInput;
 
 class ALSAAudioInput : public AudioInput {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(ALSAAudioInput)
 	public:
 		ALSAAudioInput();
 		~ALSAAudioInput();
@@ -48,7 +50,9 @@ class ALSAAudioInput : public AudioInput {
 };
 
 class ALSAAudioOutput : public AudioOutput {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(ALSAAudioOutput)
 	protected:
 	public:
 		ALSAAudioOutput();
