@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -45,7 +45,7 @@ class ASIOAudioInputRegistrar : public AudioInputRegistrar {
 		virtual AudioInput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
 		virtual void setDeviceChoice(const QVariant &, Settings &);
-		virtual bool canEcho(const QString &);
+		virtual bool canEcho(const QString &) const;
 
 };
 
@@ -60,7 +60,7 @@ const QList<audioDevice> ASIOAudioInputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-bool ASIOAudioInputRegistrar::canEcho(const QString &) {
+bool ASIOAudioInputRegistrar::canEcho(const QString &) const {
 	return true;
 }
 
@@ -125,6 +125,8 @@ void ASIOInit::initialize() {
 	if (bFound) {
 		airASIO = new ASIOAudioInputRegistrar();
 		crASIO = new ConfigRegistrar(2002, ASIOConfigDialogNew);
+	} else {
+		qWarning("ASIO: No valid devices found, disabling");
 	}
 }
 
