@@ -86,8 +86,8 @@ class AudioOutputDialog : public ConfigWidget, public Ui::AudioOutput {
 		void on_qcbLoopback_currentIndexChanged(int v);
 		void on_qsMinDistance_valueChanged(int v);
 		void on_qsMaxDistance_valueChanged(int v);
-		void on_qsRollOff_valueChanged(int v);
 		void on_qsBloom_valueChanged(int v);
+		void on_qsMaxDistVolume_valueChanged(int v);
 		void on_qcbSystem_currentIndexChanged(int);
 		void on_qcbPositional_stateChanged(int);
 };
