@@ -113,6 +113,169 @@ void AudioOutputPlayer::resizeBuffer(unsigned int newsize) {
 	}
 }
 
+AudioOutputSample::AudioOutputSample(const QString &filename, const QList<QByteArray> &packets, bool loop, unsigned int freq) : AudioOutputPlayer(filename) {
+	dsDecState=speex_decoder_init(&speex_wb_mode);
+
+	int iArg=1;
+	int err;
+
+	speex_decoder_ctl(dsDecState, SPEEX_SET_ENH, &iArg);
+	speex_decoder_ctl(dsDecState, SPEEX_GET_FRAME_SIZE, &iFrameSize);
+
+	if (freq != SAMPLE_RATE)
+		srs = speex_resampler_init(1, SAMPLE_RATE, freq, 3, &err);
+	else
+		srs = NULL;
+
+	iOutputSize = lroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(SAMPLE_RATE)));
+
+	iBufferOffset = iBufferFilled = iLastConsume = 0;
+
+	bLoop = false;
+
+	iPacketIndex = -1;
+
+	qlPackets = packets;
+
+	speex_bits_init(&sbBits);
+
+	bLoop = loop;
+}
+
+AudioOutputSample::~AudioOutputSample() {
+	speex_decoder_destroy(dsDecState);
+	speex_bits_destroy(&sbBits);
+}
+
+QList<QByteArray> AudioOutputSample::getPacketsFromFile(const QString &filename) {
+	QList<QByteArray> packets;
+
+	QFile f(filename);
+	if (! f.open(QIODevice::ReadOnly)) {
+		qWarning("AudioOutputSample: Failed to open %s for reading", qPrintable(filename));
+		return QList<QByteArray>();
+	}
+
+	QByteArray qbaSample = f.readAll();
+	f.close();
+
+	if (qbaSample.startsWith(QString::fromLatin1("OggS").toUtf8())) {
+		ogg_sync_state sync;
+		ogg_page page;
+		ogg_packet packet;
+		ogg_stream_state stream;
+		bool stream_init = false;
+		bool eos = false;
+		int speex_serialno = -1;
+		int packetno = -1;
+		bool header_ok = false;
+		int nframes = -1;
+		int extra_headers = 0;
+
+		ogg_sync_init(&sync);
+		char *data = ogg_sync_buffer(&sync, qbaSample.length());
+		memcpy(data, qbaSample.constData(), qbaSample.length());
+		ogg_sync_wrote(&sync, qbaSample.length());
+
+		while (ogg_sync_pageout(&sync, &page)==1) {
+			if (! stream_init)  {
+				ogg_stream_init(&stream, ogg_page_serialno(&page));
+				stream_init = true;
+			} else if (ogg_page_serialno(&page) != stream.serialno) {
+				ogg_stream_reset_serialno(&stream, ogg_page_serialno(&page));
+			}
+			ogg_stream_pagein(&stream, &page);
+			while (! eos && ogg_stream_packetout(&stream, &packet) == 1) {
+				if (packet.bytes >= 5 && memcmp(packet.packet, "Speex", 5)==0) {
+					speex_serialno = stream.serialno;
+				}
+				if (speex_serialno == -1 || stream.serialno != speex_serialno)
+					break;
+
+				++packetno;
+
+				if (packetno == 0) {
+					SpeexHeader *header = speex_packet_to_header(reinterpret_cast<char *>(packet.packet), packet.bytes);
+
+					if (header && header->speex_version_id == 1 && header->rate == 16000 && header->nb_channels == 1) {
+						header_ok = true;
+						extra_headers = header->extra_headers;
+						nframes = header->frames_per_packet;
+					} else {
+						break;
+					}
+				} else if (packetno <= 1 + extra_headers) {
+				} else {
+					if (packet.e_o_s)
+						eos = true;
+					packets << QByteArray(reinterpret_cast<const char *>(packet.packet), packet.bytes);
+				}
+			}
+		}
+		ogg_sync_clear(&sync);
+		if (stream_init)
+			ogg_stream_clear(&stream);
+
+		if (! header_ok) {
+			qWarning("AudioOutputSample: %s contained invalid data", qPrintable(filename));
+		}
+	} else {
+		qWarning("AudioOutputSample: %s is not an ogg file", qPrintable(filename));
+	}
+	return packets;
+}
+
+bool AudioOutputSample::needSamples(unsigned int snum) {
+	for (unsigned int i=iLastConsume;i<iBufferFilled;++i)
+		pfBuffer[i-iLastConsume]=pfBuffer[i];
+	iBufferFilled -= iLastConsume;
+
+	iLastConsume = snum;
+
+	if (iBufferFilled >= snum)
+		return bLastAlive;
+
+	float *pOut;
+	STACKVAR(float, fOut, iFrameSize);
+
+
+	while (iBufferFilled < snum) {
+		resizeBuffer(iBufferFilled + iOutputSize);
+
+		pOut = (srs) ? fOut : pfBuffer + iBufferFilled;
+
+		if (speex_bits_remaining(&sbBits) < 43) {
+			++iPacketIndex;
+			if (iPacketIndex >= qlPackets.count()) {
+				if (qlPackets.isEmpty() || !bLoop) {
+					memset(pfBuffer + iBufferFilled, 0, sizeof(float) * (snum-iBufferFilled));
+					return false;
+				} else {
+					iPacketIndex = 0;
+				}
+			}
+
+			const QByteArray &qba = qlPackets[iPacketIndex];
+
+			speex_bits_set_bit_buffer(&sbBits, reinterpret_cast<void *>(const_cast<char *>(qba.constData())), qba.length());
+			speex_bits_rewind(&sbBits);
+		}
+
+		if (speex_decode(dsDecState, &sbBits, pOut) != 0) {
+			memset(pfBuffer + iBufferFilled, 0, sizeof(float) * (snum-iBufferFilled));
+			return false;
+		}
+
+		spx_uint32_t inlen = iFrameSize;
+		spx_uint32_t outlen = iOutputSize;
+		if (srs)
+			speex_resampler_process_float(srs, 0, fOut, &inlen, pfBuffer + iBufferFilled, &outlen);
+		iBufferFilled += outlen;
+	}
+
+	return true;
+}
+
 AudioOutputSpeech::AudioOutputSpeech(ClientPlayer *player, unsigned int freq) : AudioOutputPlayer(player->qsName) {
 	p = player;
 
@@ -271,6 +434,7 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 	return bLastAlive;
 }
 
+
 AudioOutput::AudioOutput() {
 	void *ds=speex_decoder_init(&speex_wb_mode);
 	speex_decoder_ctl(ds, SPEEX_GET_FRAME_SIZE, &iFrameSize);
@@ -311,16 +475,25 @@ float AudioOutput::calcGain(float dotproduct, float distance) {
 
 
 	// No distance attenuation
-	if (g.s.fAudioRollOff < 0.01f) {
+	if (g.s.fAudioMaxDistVolume > 0.99f) {
 		att = qMin(1.0f, dotfactor + g.s.fAudioBloom);
 	} else if (distance < g.s.fAudioMinDistance) {
 		float bloomfac = g.s.fAudioBloom * (1.0f - distance/g.s.fAudioMinDistance);
 
 		att = qMin(1.0f, bloomfac + dotfactor);
 	} else {
-		if (distance > g.s.fAudioMaxDistance)
-			distance = g.s.fAudioMaxDistance;
-		float datt = g.s.fAudioMinDistance / (g.s.fAudioMinDistance + (distance-g.s.fAudioMinDistance)*g.s.fAudioRollOff);
+		float datt;
+
+		if (distance >= g.s.fAudioMaxDistance) {
+			datt = g.s.fAudioMaxDistVolume;
+		} else {
+			float mvol = g.s.fAudioMaxDistVolume;
+			if (mvol < 0.01f)
+				mvol = 0.01f;
+
+			float drel = (distance-g.s.fAudioMinDistance) / (g.s.fAudioMaxDistance - g.s.fAudioMinDistance);
+			datt = powf(10.0f, log10f(mvol) * drel);
+		}
 
 		att = datt * dotfactor;
 	}
@@ -350,6 +523,21 @@ AudioSine *AudioOutput::playSine(float hz, float i, unsigned int frames, float v
 	return as;
 }
 
+AudioOutputSample *AudioOutput::playSample(const QString &filename, bool loop) {
+	QList<QByteArray> packets = AudioOutputSample::getPacketsFromFile(filename);
+	if (packets.isEmpty())
+		return NULL;
+
+	while ((iMixerFreq == 0) && isRunning()) {}
+
+	qrwlOutputs.lockForWrite();
+	AudioOutputSample *aos = new AudioOutputSample(filename, packets, loop, iMixerFreq);
+	qmOutputs.insert(NULL, aos);
+	qrwlOutputs.unlock();
+	return aos;
+
+}
+
 void AudioOutput::addFrameToBuffer(ClientPlayer *player, const QByteArray &qbaPacket, unsigned int iSeq) {
 	if (iChannels == 0)
 		return;
@@ -614,8 +802,9 @@ bool AudioOutput::mix(void *outbuff, unsigned int nsamp) {
 					/*
 										qWarning("%d: Pos %f %f %f : Dot %f Len %f Str %f", s, speaker[s*3+0], speaker[s*3+1], speaker[s*3+2], dot, len, str);
 					*/
-					for (unsigned int i=0;i<nsamp;++i)
-						o[i*nchan] += pfBuffer[i] * (old + inc*static_cast<float>(i));
+					if ((old >= 0.00000001f) || (str >= 0.00000001f))
+						for (unsigned int i=0;i<nsamp;++i)
+							o[i*nchan] += pfBuffer[i] * (old + inc*static_cast<float>(i));
 				}
 			} else {
 				for (unsigned int s=0;s<nchan;++s) {
