@@ -69,35 +69,38 @@ void AudioBar::paintEvent(QPaintEvent *) {
 	float scale = static_cast<float>(width()) / static_cast<float>(iMax - iMin);
 	int h = height();
 
-	p.scale(scale, h);
+	int val = lroundf(iValue * scale);
+	int below = lroundf(iBelow * scale);
+	int above = lroundf(iAbove * scale);
+	int max = lroundf(iMax * scale);
+	int min = lroundf(iMin * scale);
+	int peak = lroundf(iPeak * scale);
 
-	p.fillRect(QRect(0,0, 10, 10), Qt::blue);
-
-	if (iValue <= iBelow) {
-		p.fillRect(0, 0, iValue-1, 1, qcBelow);
-		p.fillRect(iValue, 0, iBelow-iValue, 1, qcBelow.darker(300));
-		p.fillRect(iBelow, 0, iAbove-iBelow, 1, qcInside.darker(300));
-		p.fillRect(iAbove, 0, iMax-iAbove, 1, qcAbove.darker(300));
-	} else if (iValue <= iAbove) {
-		p.fillRect(0, 0, iBelow, 1, qcBelow);
-		p.fillRect(iBelow, 0, iValue-iBelow, 1, qcInside);
-		p.fillRect(iValue, 0, iAbove-iValue, 1, qcInside.darker(300));
-		p.fillRect(iAbove, 0, iMax-iAbove, 1, qcAbove.darker(300));
+	if (val <= below) {
+		p.fillRect(0, 0, val, h, qcBelow);
+		p.fillRect(val, 0, below-val, h, qcBelow.darker(300));
+		p.fillRect(below, 0, above-below, h, qcInside.darker(300));
+		p.fillRect(above, 0, max-above, h, qcAbove.darker(300));
+	} else if (val <= above) {
+		p.fillRect(0, 0, below, h, qcBelow);
+		p.fillRect(below, 0, val-below, h, qcInside);
+		p.fillRect(val, 0, above-val, h, qcInside.darker(300));
+		p.fillRect(above, 0, max-above, h, qcAbove.darker(300));
 	} else {
-		p.fillRect(0, 0, iBelow, 1, qcBelow);
-		p.fillRect(iBelow, 0, iAbove-iBelow, 1, qcInside);
-		p.fillRect(iAbove, 0, iValue-iAbove, 1, qcAbove);
-		p.fillRect(iValue, 0, iMax-iValue, 1, qcAbove.darker(300));
+		p.fillRect(0, 0, below, h, qcBelow);
+		p.fillRect(below, 0, above-below, h, qcInside);
+		p.fillRect(above, 0, val-above, h, qcAbove);
+		p.fillRect(val, 0, max-val, h, qcAbove.darker(300));
 	}
 
-	if ((iPeak >= iMin) && (iPeak <= iMax))  {
-		if (iPeak <= iBelow)
+	if ((peak >= min) && (peak <= max))  {
+		if (peak <= below)
 			p.setPen(qcBelow.lighter(150));
-		else if (iPeak <= iAbove)
+		else if (peak <= above)
 			p.setPen(qcInside.lighter(150));
 		else
 			p.setPen(qcAbove.lighter(150));
-		p.drawLine(iPeak, 0, iPeak, 1);
+		p.drawLine(peak, 0, peak, h);
 	}
 
 }
