@@ -301,7 +301,7 @@ CompletablePage *AudioWizard::positionalPage() {
 	fX = fY = 0.0f;
 	qgsScene = NULL;
 	qgiSource = NULL;
-	asSource = NULL;
+	aosSource = NULL;
 	qgvView = new QGraphicsView(qwpage);
 	qgvView->scale(1.0f, -1.0f);
 	qgvView->viewport()->installEventFilter(this);
@@ -375,7 +375,12 @@ CompletablePage *AudioWizard::triggerPage() {
 
 	skwPTT = new ShortcutKeyWidget(qwpage);
 	skwPTT->setObjectName(QLatin1String("PTTKey"));
-	skwPTT->setShortcut(g.s.qmShortcuts.value(g.mw->gsPushTalk->id()));
+	foreach(const Shortcut &s, g.s.qlShortcuts) {
+		if (s.iIndex == g.mw->gsPushTalk->idx) {
+			skwPTT->setShortcut(s.qlButtons);
+			break;
+		}
+	}
 	h->addWidget(skwPTT);
 
 	v->addLayout(h);
@@ -614,7 +619,7 @@ void AudioWizard::showPage(int) {
 	AudioOutputPtr ao = g.ao;
 	if (ao)
 		ao->wipe();
-	asSource = NULL;
+	aosSource = NULL;
 
 	g.bPosTest = false;
 
@@ -624,7 +629,7 @@ void AudioWizard::showPage(int) {
 		g.s.bMute = true;
 	} else if (cp == qwpDeviceTuning) {
 		g.s.bMute = true;
-		ao->playSine(0.0f, 0.0f, 0xfffffff, 0.5f);
+		playChord();
 	} else if (cp == qwpPositional) {
 		fX = fY = 0.0f;
 		g.s.bMute = true;
@@ -653,16 +658,16 @@ int AudioWizard::nextId() const {
 
 void AudioWizard::playChord() {
 	AudioOutputPtr ao = g.ao;
-	if (! ao || asSource)
+	if (! ao || aosSource)
 		return;
-	asSource = ao->playSine(100.0f, 0.0001f, 0xfffffff, 0.5f);
+	aosSource = ao->playSample(QLatin1String("skin:wb_male.spx"), true);
 }
 
 void AudioWizard::restartAudio() {
 	boost::weak_ptr<AudioInput> wai(g.ai);
 	boost::weak_ptr<AudioOutput> wao(g.ao);
 
-	asSource = NULL;
+	aosSource = NULL;
 
 	g.ai.reset();
 	g.ao.reset();
@@ -712,7 +717,30 @@ void AudioWizard::accept() {
 	g.s.bMute = sOldSettings.bMute;
 	g.s.bDeaf = sOldSettings.bDeaf;
 	g.s.lmLoopMode = Settings::None;
-	g.s.qmShortcuts.insert(g.mw->gsPushTalk->id(), skwPTT->qlButtons);
+
+	const QList<QVariant> &buttons = skwPTT->getShortcut();
+	QList<Shortcut> ql;
+	bool found = false;
+	foreach(Shortcut s, g.s.qlShortcuts) {
+		if (s.iIndex == g.mw->gsPushTalk->idx) {
+			if (buttons.isEmpty())
+				continue;
+			else if (! found) {
+				s.qlButtons = buttons;
+				found = true;
+			}
+		}
+		ql << s;
+	}
+	if (! found && ! buttons.isEmpty()) {
+		Shortcut s;
+		s.iIndex = g.mw->gsPushTalk->idx;
+		s.bSuppress = false;
+		s.qlButtons = buttons;
+		ql << s;
+	}
+	g.s.qlShortcuts = ql;
+
 	g.s.bUsage = qcbUsage->isChecked();
 	g.bEchoTest = false;
 	g.bPosTest = false;
@@ -794,9 +822,11 @@ void AudioWizard::on_Ticker_timeout() {
 		}
 
 		qgiSource->setPos(xp, yp);
-		asSource->fPos[0] = xp;
-		asSource->fPos[1] = 0;
-		asSource->fPos[2] = yp;
+		if (aosSource) {
+			aosSource->fPos[0] = xp;
+			aosSource->fPos[1] = 0;
+			aosSource->fPos[2] = yp;
+		}
 	}
 }
 
