@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -36,7 +36,9 @@
 #include "Settings.h"
 
 class ConfigWidget : public QWidget {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(ConfigWidget)
 	protected:
 		void loadSlider(QSlider *, int);
 		void loadCheckBox(QAbstractButton *, bool);
@@ -59,6 +61,8 @@ typedef ConfigWidget *(*ConfigWidgetNew)(Settings &st);
 
 class ConfigRegistrar {
 		friend class ConfigDialog;
+	private:
+		Q_DISABLE_COPY(ConfigRegistrar)
 	protected:
 		int iPriority;
 		static QMap<int, ConfigWidgetNew> *c_qmNew;
@@ -68,7 +72,9 @@ class ConfigRegistrar {
 };
 
 class ConfigDialog : public QDialog, public Ui::ConfigDialog {
-		Q_OBJECT;
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(ConfigDialog)
 	protected:
 		QHash<QListWidgetItem *, ConfigWidget *> qhWidgets;
 		QHash<ConfigWidget *, QWidget *> qhPages;
@@ -77,11 +83,11 @@ class ConfigDialog : public QDialog, public Ui::ConfigDialog {
 		Settings s;
 	public:
 		ConfigDialog(QWidget *p = NULL);
+		~ConfigDialog();
 	public slots:
-		void on_qlwIcons_currentItemChanged(QListWidgetItem *current, QListWidgetItem *previous);
 		void on_pageButtonBox_clicked(QAbstractButton *);
 		void on_dialogButtonBox_clicked(QAbstractButton *);
-		void on_qcbExpert_clicked(bool);
+		void updateExpert(bool);
 		void apply();
 		void accept();
 };
