@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -34,14 +34,22 @@
 #include "mumble_pch.hpp"
 
 class MumbleDBus : public QDBusAbstractAdaptor {
+	private:
 		Q_OBJECT
-		Q_CLASSINFO("D-Bus Interface", "net.sourceforge.mumble.Mumble");
+		Q_CLASSINFO("D-Bus Interface", "net.sourceforge.mumble.Mumble")
+		Q_DISABLE_COPY(MumbleDBus)
+		Q_PROPERTY(bool mute READ isSelfMuted WRITE setSelfMuted)
+		Q_PROPERTY(bool deaf READ isSelfDeaf WRITE setSelfDeaf)
 	public:
 		MumbleDBus(QObject *parent);
 	public slots:
 		void openUrl(const QString &url, const QDBusMessage &);
 		void getCurrentUrl(const QDBusMessage &);
 		void focus();
+		void setSelfMuted(bool mute);
+		void setSelfDeaf(bool deafen);
+		bool isSelfMuted();
+		bool isSelfDeaf();
 };
 
 #else
