@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -34,7 +34,9 @@
 #include "mumble_pch.hpp"
 
 class Database : public QObject {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(Database)
 	public:
 		Database();
 		static const QString getDigest(const QString &hostname, unsigned short port);
