@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -55,10 +55,10 @@ struct InputDevice {
 };
 
 class GlobalShortcutWin : public GlobalShortcutEngine {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(GlobalShortcutWin)
 	public:
-		QTimer *timer;
-
 		LPDIRECTINPUT8 pDI;
 		QHash<GUID, InputDevice *> qhInputDevices;
 		HHOOK hhMouse, hhKeyboard;
