@@ -33,6 +33,7 @@
 
 #include "mumble_pch.hpp"
 #include "ConfigDialog.h"
+#include "AudioOutput.h"
 #include "ui_Log.h"
 
 class TextToSpeech;
@@ -42,6 +43,7 @@ class LogConfig : public ConfigWidget, public Ui::LogConfig {
 		Q_OBJECT
 		Q_DISABLE_COPY(LogConfig)
 	public:
+		enum Column { ColMessage, ColConsole, ColNotification, ColTTS, ColStaticSound, ColStaticSoundPath };
 		LogConfig(Settings &st);
 		virtual QString title() const;
 		virtual QIcon icon() const;
@@ -50,6 +52,11 @@ class LogConfig : public ConfigWidget, public Ui::LogConfig {
 		virtual void save() const;
 		virtual void load(const Settings &);
 		virtual bool expert(bool);
+
+		void on_qtwMessages_itemChanged(QTreeWidgetItem*, int);
+		void on_qtwMessages_itemClicked(QTreeWidgetItem*, int);
+		void on_qtwMessages_itemDoubleClicked(QTreeWidgetItem*, int);
+		void browseForSpxFile();
 };
 
 class Log : public QObject {
@@ -65,6 +72,7 @@ class Log : public QObject {
 		QHash<MsgType, int> qmIgnore;
 		static const char *msgNames[];
 		TextToSpeech *tts;
+		unsigned int uiLastId;
 	public:
 		Log(QObject *p = NULL);
 		QString msgName(MsgType t) const;
@@ -74,6 +82,18 @@ class Log : public QObject {
 		void log(MsgType t, const QString &console, const QString &terse=QString());
 };
 
+class LogDocument : public QTextDocument {
+		Q_OBJECT
+	public:
+		QVariant qvVariant;
+		int iType;
+		QUrl quUrl;
+		LogDocument(QObject *p);
+		QVariant loadResource(int, const QUrl &);
+	public slots:
+		void finished();
+};
+
 #else
 class Log;
 #endif
