@@ -125,7 +125,7 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	qApp->setWindowIcon(qiIcon);
 #endif
 
-	Channel::add(0, tr("Root"), NULL);
+	Channel::add(0, tr("Root"));
 
 	aclEdit = NULL;
 	banEdit = NULL;
@@ -148,22 +148,34 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	foreach(QAction *a, qla)
 		a->setShortcutContext(Qt::ApplicationShortcut);
 
+	on_qmServer_aboutToShow();
+	on_qmChannel_aboutToShow();
+	on_qmPlayer_aboutToShow();
+	on_qmConfig_aboutToShow();
+
 	setOnTop(g.s.bAlwaysOnTop);
 }
 
 void MainWindow::createActions() {
 	int idx = 1;
-	gsPushTalk=new GlobalShortcut(this, idx++, tr("Push-to-Talk", "Global Shortcut"));
+	gsPushTalk=new GlobalShortcut(this, idx++, tr("Push-to-Talk", "Global Shortcut"), false);
 	gsPushTalk->setObjectName(QLatin1String("PushToTalk"));
+	gsPushTalk->qsToolTip = tr("Push and hold this button to send voice.", "Global Shortcut");
+	gsPushTalk->qsWhatsThis = tr("This configures the push-to-talk button, and as long as you hold this button down, you will transmit voice.", "Global Shortcut");
+
 
 	gsResetAudio=new GlobalShortcut(this, idx++, tr("Reset Audio Processor", "Global Shortcut"));
 	gsResetAudio->setObjectName(QLatin1String("ResetAudio"));
 
-	gsMuteSelf=new GlobalShortcut(this, idx++, tr("Toggle Mute Self", "Global Shortcut"));
+	gsMuteSelf=new GlobalShortcut(this, idx++, tr("Toggle Mute Self", "Global Shortcut"), false);
 	gsMuteSelf->setObjectName(QLatin1String("MuteSelf"));
+	gsMuteSelf->qsToolTip = tr("Toggle self-mute status.", "Global Shortcut");
+	gsMuteSelf->qsWhatsThis = tr("This will toggle your muted status. If you toggle this off, you will also disable self-deafen.", "Global Shortcut");
 
-	gsDeafSelf=new GlobalShortcut(this, idx++, tr("Toggle Deafen Self", "Global Shortcut"));
+	gsDeafSelf=new GlobalShortcut(this, idx++, tr("Toggle Deafen Self", "Global Shortcut"), false);
 	gsDeafSelf->setObjectName(QLatin1String("DeafSelf"));
+	gsDeafSelf->qsToolTip = tr("Toggle self-deafen status.", "Global Shortcut");
+	gsDeafSelf->qsWhatsThis = tr("This will toggle your deafen status. If you toggle this on, you will also enable self-mute.", "Global Shortcut");
 
 	gsUnlink=new GlobalShortcut(this, idx++, tr("Unlink Plugin", "Global Shortcut"));
 	gsUnlink->setObjectName(QLatin1String("UnlinkPlugin"));
@@ -174,17 +186,17 @@ void MainWindow::createActions() {
 	GlobalShortcut *gs;
 
 	gs = new GlobalShortcut(this, idx++, tr("Chan Parent", "Global Shortcut"));
-	gs->setData(0);
+	gs->data = 0;
 	connect(gs, SIGNAL(triggered(bool)), this, SLOT(pushLink(bool)));
 
 	for (int i = 1; i< 10;i++) {
 		gs = new GlobalShortcut(this, idx++, tr("Chan Sub#%1", "Global Shortcut").arg(i));
-		gs->setData(i);
+		gs->data = i;
 		connect(gs, SIGNAL(triggered(bool)), this, SLOT(pushLink(bool)));
 	}
 
 	gs = new GlobalShortcut(this, idx++, tr("Chan All Subs", "Global Shortcut"));
-	gs->setData(10);
+	gs->data = 10;
 	connect(gs, SIGNAL(triggered(bool)), this, SLOT(pushLink(bool)));
 
 	gsPushMute=new GlobalShortcut(this, idx++, tr("Push-to-Mute", "Global Shortcut"));
@@ -193,8 +205,10 @@ void MainWindow::createActions() {
 	gsMetaChannel=new GlobalShortcut(this, idx++, tr("Join Channel", "Global Shortcut"));
 	gsMetaChannel->setObjectName(QLatin1String("MetaChannel"));
 
-	gsToggleOverlay=new GlobalShortcut(this, idx++, tr("Toggle Overlay", "Global Shortcut"));
+	gsToggleOverlay=new GlobalShortcut(this, idx++, tr("Toggle Overlay", "Global Shortcut"), false);
 	gsToggleOverlay->setObjectName(QLatin1String("ToggleOverlay"));
+	gsToggleOverlay->qsToolTip = tr("Toggle state of in-game overlay.", "Global Shortcut");
+	gsToggleOverlay->qsWhatsThis = tr("This will switch the states of the ingame overlay between showing everybody, just the players who are talking, and nobody.", "Global Shortcut");
 	connect(gsToggleOverlay, SIGNAL(down()), g.o, SLOT(toggleShow()));
 
 	gsAltTalk=new GlobalShortcut(this, idx++, tr("Alt Push-to-Talk", "Global Shortcut"));
@@ -213,10 +227,6 @@ void MainWindow::createActions() {
 	qstiIcon->setToolTip(tr("Mumble"));
 	qstiIcon->setObjectName(QLatin1String("Icon"));
 
-	qmTray = new QMenu(this);
-	qmTray->addAction(tr("&Quit"), this, SLOT(on_qaQuit_triggered()));
-	qstiIcon->setContextMenu(qmTray);
-	updateTrayIcon();
 #ifndef Q_OS_MAC
 	qstiIcon->show();
 #endif
@@ -230,9 +240,11 @@ void MainWindow::setupGui()  {
 	QMenu *qmWindow = new QMenu(tr("&Window"), this);
 	menubar->insertMenu(qmHelp->menuAction(), qmWindow);
 	qmWindow->addAction(tr("Minimize"), this, SLOT(showMinimized()), QKeySequence(tr("Ctrl+M")));
-	qmWindow->addAction(tr("Close"), this, SLOT(on_qaQuit_triggered()), QKeySequence(tr("Ctrl+W")));
 #endif
 
+	LogDocument *ld = new LogDocument(qteLog);
+	qteLog->setDocument(ld);
+
 	qteLog->document()->setDefaultStyleSheet(qApp->styleSheet());
 
 	pmModel = new PlayerModel(qtvPlayers);
@@ -275,11 +287,22 @@ void MainWindow::setupGui()  {
 	qtvPlayers->header()->restoreState(g.s.qbaHeaderState);
 
 	setupView(false);
+
+	qmTray = new QMenu(this);
+	qmTray->addAction(qaQuit);
+	qmTray->addAction(qaHelpAbout);
+	qmTray->addSeparator();
+	qmTray->addAction(qaAudioMute);
+	qmTray->addAction(qaAudioDeaf);
+	qstiIcon->setContextMenu(qmTray);
+	updateTrayIcon();
 }
 
 MainWindow::~MainWindow() {
+	delete qdwLog->titleBarWidget();
 	delete pmModel;
 	delete qtvPlayers;
+	delete Channel::get(0);
 }
 
 void MainWindow::msgBox(QString msg) {
@@ -327,6 +350,8 @@ void MainWindow::on_qtvPlayers_customContextMenuRequested(const QPoint &mpos) {
 	QModelIndex idx = qtvPlayers->indexAt(mpos);
 	if (! idx.isValid())
 		idx = qtvPlayers->currentIndex();
+	else
+		qtvPlayers->setCurrentIndex(idx);
 	Player *p = pmModel->getPlayer(idx);
 
 	if (p) {
@@ -544,6 +569,23 @@ void MainWindow::on_Reconnect_timeout() {
 	g.sh->start(QThread::TimeCriticalPriority);
 }
 
+void MainWindow::on_qmServer_aboutToShow() {
+	qmServer->clear();
+	qmServer->addAction(qaServerConnect);
+	qmServer->addAction(qaServerDisconnect);
+	qmServer->addAction(qaServerBanList);
+	qmServer->addAction(qaServerInformation);
+	qmServer->addSeparator();
+	qmServer->addAction(qaQuit);
+
+	if (! qlServerActions.isEmpty()) {
+		qmServer->addSeparator();
+		foreach(QAction *a, qlServerActions)
+			qmServer->addAction(a);
+	}
+}
+
+
 void MainWindow::on_qaServerDisconnect_triggered() {
 	if (qtReconnect->isActive())
 		qtReconnect->stop();
@@ -631,6 +673,12 @@ void MainWindow::on_qmPlayer_aboutToShow() {
 		qmPlayer->addMenu(qmHelp);
 	}
 
+	if (! qlPlayerActions.isEmpty()) {
+		qmPlayer->addSeparator();
+		foreach(QAction *a, qlPlayerActions)
+			qmPlayer->addAction(a);
+	}
+
 	if (! p) {
 		qaPlayerKick->setEnabled(false);
 		qaPlayerBan->setEnabled(false);
@@ -776,6 +824,7 @@ void MainWindow::on_qmChannel_aboutToShow() {
 	qmChannel->addAction(qaChannelRemove);
 	qmChannel->addAction(qaChannelACL);
 	qmChannel->addAction(qaChannelRename);
+	qmChannel->addAction(qaChannelDescUpdate);
 	qmChannel->addSeparator();
 	qmChannel->addAction(qaChannelLink);
 	qmChannel->addAction(qaChannelUnlink);
@@ -792,13 +841,20 @@ void MainWindow::on_qmChannel_aboutToShow() {
 		qmChannel->addMenu(qmHelp);
 	}
 
-	bool add, remove, acl, rename, link, unlink, unlinkall, msg;
-	add = remove = acl = rename = link = unlink = unlinkall = msg = false;
+	if (! qlChannelActions.isEmpty()) {
+		qmChannel->addSeparator();
+		foreach(QAction *a, qlChannelActions)
+			qmChannel->addAction(a);
+	}
+
+	bool add, remove, acl, rename, descUpdate, link, unlink, unlinkall, msg;
+	add = remove = acl = rename = descUpdate = link = unlink = unlinkall = msg = false;
 
 	if (g.uiSession != 0) {
 		add = true;
 		acl = true;
 		msg = true;
+		descUpdate = true;
 
 		Channel *c = pmModel->getChannel(idx);
 		Channel *home = ClientPlayer::get(g.uiSession)->cChannel;
@@ -822,6 +878,7 @@ void MainWindow::on_qmChannel_aboutToShow() {
 	qaChannelRemove->setEnabled(remove);
 	qaChannelACL->setEnabled(acl);
 	qaChannelRename->setEnabled(rename);
+	qaChannelDescUpdate->setEnabled(descUpdate);
 	qaChannelLink->setEnabled(link);
 	qaChannelUnlink->setEnabled(unlink);
 	qaChannelUnlinkAll->setEnabled(unlinkall);
@@ -890,6 +947,33 @@ void MainWindow::on_qaChannelRename_triggered() {
 	}
 }
 
+void MainWindow::on_qaChannelDescUpdate_triggered() {
+	Channel *c = pmModel->getChannel(qtvPlayers->currentIndex());
+	if (! c)
+		return;
+
+	int id = c->iId;
+
+	TextMessage tm;
+	tm.setWindowTitle(tr("Change description of channel %1").arg(c->qsName));
+
+	const QString html = QTextDocumentFragment::fromPlainText(c->qsDesc).toHtml();
+
+	tm.qteEdit->setText(html);
+	int res = tm.exec();
+
+	c = Channel::get(id);
+	if (!c)
+		return;
+
+	if (res==QDialog::Accepted) {
+		MessageChannelDescUpdate mcdu;
+		mcdu.iId = id;
+		mcdu.qsDesc = tm.message();
+		g.sh->sendMessage(&mcdu);
+	}
+}
+
 void MainWindow::on_qaChannelACL_triggered() {
 	Channel *c = pmModel->getChannel(qtvPlayers->currentIndex());
 	int id = c ? c->iId : 0;
@@ -1002,7 +1086,12 @@ void MainWindow::on_qaAudioReset_triggered() {
 }
 
 void MainWindow::on_qaAudioMute_triggered() {
+	AudioInputPtr ai = g.ai;
+	if (ai)
+		ai->tIdle.restart();
+
 	g.s.bMute = qaAudioMute->isChecked();
+
 	if (! g.s.bMute && g.s.bDeaf) {
 		g.s.bDeaf = false;
 		qaAudioDeaf->setChecked(false);
@@ -1022,6 +1111,10 @@ void MainWindow::on_qaAudioMute_triggered() {
 }
 
 void MainWindow::on_qaAudioDeaf_triggered() {
+	AudioInputPtr ai = g.ai;
+	if (ai)
+		ai->tIdle.restart();
+
 	g.s.bDeaf = qaAudioDeaf->isChecked();
 	if (g.s.bDeaf && ! g.s.bMute) {
 		g.s.bMute = true;
@@ -1056,7 +1149,8 @@ void MainWindow::on_qaAudioUnlink_triggered() {
 
 void MainWindow::on_qaConfigDialog_triggered() {
 	ConfigDialog *dlg= new ConfigDialog(this);
-	dlg->exec();
+	if (dlg->exec() == QDialog::Accepted)
+		setOnTop(g.s.bAlwaysOnTop);
 	delete dlg;
 }
 
@@ -1164,7 +1258,7 @@ void MainWindow::pushLink(bool down) {
 	GlobalShortcut *gs = qobject_cast<GlobalShortcut *>(sender());
 	if (! gs)
 		return;
-	int idx = gs->data().toInt();
+	int idx = gs->data.toInt();
 	Channel *home = ClientPlayer::get(g.uiSession)->cChannel;
 
 	Channel *target = NULL;
@@ -1260,6 +1354,18 @@ void MainWindow::serverDisconnected(QString reason) {
 		banEdit = NULL;
 	}
 
+	QSet<QAction *> qs;
+	qs += qlServerActions.toSet();
+	qs += qlChannelActions.toSet();
+	qs += qlPlayerActions.toSet();
+
+	foreach(QAction *a, qs)
+		delete a;
+
+	qlServerActions.clear();
+	qlChannelActions.clear();
+	qlPlayerActions.clear();
+
 	pmModel->removeAll();
 	qtvPlayers->setRowHidden(0, QModelIndex(), true);
 
@@ -1387,3 +1493,18 @@ void MainWindow::on_qteLog_highlighted(const QUrl &url) {
 	else
 		QToolTip::showText(QCursor::pos(), url.toString(), qteLog, QRect());
 }
+
+void MainWindow::context_triggered() {
+	QAction *a = qobject_cast<QAction *>(sender());
+
+	Channel *c = pmModel->getChannel(qtvPlayers->currentIndex());
+	ClientPlayer *p = pmModel->getPlayer(qtvPlayers->currentIndex());
+
+	MessageContextAction mca;
+	mca.uiSession = g.uiSession;
+	mca.qsAction = a->data().toString();
+	mca.uiVictim = p ? p->uiSession : 0;
+	mca.iChannel = c ? c->iId : -1;
+
+	g.sh->sendMessage(&mca);
+}
