@@ -185,6 +185,12 @@ void MainWindow::msgPlayerMove(Connection *, MessagePlayerMove *msg) {
 	MSG_INIT;
 	VICTIM_INIT;
 
+	Channel *c = Channel::get(msg->iChannelId);
+	if (!c) {
+		qWarning("MessagePlayerMove for unknown channel.");
+		c = Channel::get(0);
+	}
+
 	bool log = true;
 	if ((msg->uiVictim == g.uiSession) && (msg->uiSession == msg->uiVictim))
 		log = false;
@@ -194,26 +200,26 @@ void MainWindow::msgPlayerMove(Connection *, MessagePlayerMove *msg) {
 	QString pname = pDst->qsName;
 	QString admin = pSrc ? pSrc->qsName : QLatin1String("server");
 
-	if (log && (pDst->cChannel == ClientPlayer::get(g.uiSession)->cChannel)) {
-		if (pDst == pSrc || (!pSrc))
-			g.l->log(Log::ChannelJoin, MainWindow::tr("%1 left channel.").arg(pname));
-		else
-			g.l->log(Log::ChannelJoin, MainWindow::tr("%1 moved out by %2.").arg(pname).arg(admin));
-	}
+	Channel *old = pDst->cChannel;
 
-	Channel *c = Channel::get(msg->iChannelId);
-	if (!c) {
-		qWarning("MessagePlayerMove for unknown channel.");
-		c = Channel::get(0);
+	if (log) {
+		if (g.uiSession == msg->uiVictim) {
+			g.l->log(Log::ChannelJoin, MainWindow::tr("You were moved to %1 by %2.").arg(c->qsName).arg(admin));
+		} else if (pDst->cChannel == ClientPlayer::get(g.uiSession)->cChannel) {
+			if (pDst == pSrc || (!pSrc))
+				g.l->log(Log::ChannelLeave, MainWindow::tr("%1 left channel.").arg(pname));
+			else
+				g.l->log(Log::ChannelLeave, MainWindow::tr("%1 moved out by %2 to %3.").arg(pname).arg(admin).arg(c->qsName));
+		}
 	}
 
 	pmModel->movePlayer(pDst, c);
 
-	if (log && (pDst->cChannel == ClientPlayer::get(g.uiSession)->cChannel)) {
+	if (log && (g.uiSession != msg->uiVictim) && (pDst->cChannel == ClientPlayer::get(g.uiSession)->cChannel)) {
 		if (pDst == pSrc || (!pSrc))
-			g.l->log(Log::ChannelLeave, MainWindow::tr("%1 entered channel.").arg(pname));
+			g.l->log(Log::ChannelJoin, MainWindow::tr("%1 entered channel.").arg(pname));
 		else
-			g.l->log(Log::ChannelLeave, MainWindow::tr("%1 moved in by %2.").arg(pname).arg(admin));
+			g.l->log(Log::ChannelJoin, MainWindow::tr("%1 moved in by %2 from %3.").arg(pname).arg(admin).arg(old->qsName));
 	}
 }
 
@@ -274,6 +280,11 @@ void MainWindow::msgChannelRename(Connection *, MessageChannelRename *msg) {
 		pmModel->renameChannel(c, msg->qsName);
 }
 
+void MainWindow::msgChannelDescUpdate(Connection *, MessageChannelDescUpdate *msg) {
+	Channel *c = Channel::get(msg->iId);
+	if (c) c->qsDesc=msg->qsDesc;
+}
+
 void MainWindow::msgChannelLink(Connection *, MessageChannelLink *msg) {
 	Channel *c = Channel::get(msg->iId);
 	if (!c)
@@ -387,3 +398,18 @@ void MainWindow::msgCryptSync(Connection *, MessageCryptSync *msg) {
 		memcpy(c->csCrypt.decrypt_iv, msg->qbaNonce.constData(), AES_BLOCK_SIZE);
 	}
 }
+
+void MainWindow::msgContextAction(Connection *, MessageContextAction *) {
+}
+
+void MainWindow::msgContextAddAction(Connection *, MessageContextAddAction *msg) {
+	QAction *a = new QAction(msg->qsText, g.mw);
+	a->setData(msg->qsAction);
+	connect(a, SIGNAL(triggered()), this, SLOT(context_triggered()));
+	if (msg->ctx & MessageContextAddAction::CtxServer)
+		qlServerActions.append(a);
+	if (msg->ctx & MessageContextAddAction::CtxPlayer)
+		qlPlayerActions.append(a);
+	if (msg->ctx & MessageContextAddAction::CtxChannel)
+		qlChannelActions.append(a);
+}
