@@ -204,6 +204,7 @@ PlayerModel::~PlayerModel() {
 	removeAll();
 	Q_ASSERT(ModelItem::c_qhPlayers.count() == 0);
 	Q_ASSERT(ModelItem::c_qhChannels.count() == 1);
+	delete miRoot;
 }
 
 
@@ -752,7 +753,7 @@ void PlayerModel::renameChannel(Channel *c, const QString &name) {
 
 
 Channel *PlayerModel::addChannel(int id, Channel *p, const QString &name) {
-	Channel *c = Channel::add(id, name, NULL);
+	Channel *c = Channel::add(id, name);
 
 	if (! c)
 		return NULL;
