@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -37,7 +37,9 @@
 struct PluginInfo;
 
 class PluginConfig : public ConfigWidget, public Ui::PluginConfig {
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(PluginConfig)
 	protected:
 		void refillPluginList();
 		QHash<QListWidgetItem *, PluginInfo *> qhInfos;
@@ -56,7 +58,9 @@ class PluginConfig : public ConfigWidget, public Ui::PluginConfig {
 
 class Plugins : public QObject {
 		friend class PluginConfig;
+	private:
 		Q_OBJECT
+		Q_DISABLE_COPY(Plugins)
 	protected:
 		QMutex qmPlugins;
 		QList<PluginInfo *> qlPlugins;
