@@ -36,6 +36,7 @@
 #include "mumble_pch.hpp"
 #include "Timer.h"
 
+
 class Connection;
 class Message;
 
@@ -59,6 +60,11 @@ class ServerHandler : public QThread {
 		QString qsDigest;
 		unsigned short usPort;
 
+#ifdef Q_OS_WIN
+		HANDLE hQoS;
+		DWORD dwFlowUDP;
+#endif
+
 		QHostAddress qhaRemote;
 		QUdpSocket *qusUdp;
 		QMutex qmUdp;
