@@ -33,6 +33,10 @@
 #include "Global.h"
 #include "AudioInput.h"
 
+bool Shortcut::operator <(const Shortcut &other) const {
+	return (iIndex < other.iIndex);
+}
+
 Settings::Settings() {
 	atTransmit = VAD;
 	bTransmitPosition = false;
@@ -87,9 +91,9 @@ Settings::Settings() {
 
 	bPositionalAudio = false;
 	bPositionalHeadphone = false;
-	fAudioMinDistance = 10.0f;
-	fAudioMaxDistance = 50.0f;
-	fAudioRollOff = 0.15f;
+	fAudioMinDistance = 1.0f;
+	fAudioMaxDistance = 15.0f;
+	fAudioMaxDistVolume = 0.15f;
 	fAudioBloom = 0.5f;
 
 	bOverlayEnable = true;
@@ -132,7 +136,10 @@ Settings::Settings() {
 	dMaxPacketDelay = 0.0f;
 
 	for (int i=Log::firstMsgType;i<=Log::lastMsgType;++i)
-		qmMessages.insert(i, Settings::LogConsole | Settings::LogTTS);
+		qmMessages.insert(i, Settings::LogConsole | Settings::LogBalloon | Settings::LogTTS);
+
+	for (int i=Log::firstMsgType;i<=Log::lastMsgType;++i)
+		qmMessageSounds.insert(i, QString());
 
 	qmMessages[Log::DebugInfo] = Settings::LogConsole;
 	qmMessages[Log::Information] = Settings::LogConsole;
@@ -197,7 +204,7 @@ void Settings::load() {
 	SAVELOAD(iIdleTime, "audio/idletime");
 	SAVELOAD(fAudioMinDistance, "audio/mindistance");
 	SAVELOAD(fAudioMaxDistance, "audio/maxdistance");
-	SAVELOAD(fAudioRollOff, "audio/rolloff");
+	SAVELOAD(fAudioMaxDistVolume, "audio/maxdistancevolume");
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
 	SAVELOAD(bPositionalAudio, "audio/positional");
@@ -291,8 +298,15 @@ void Settings::load() {
 	int nshorts = g.qs->beginReadArray(QLatin1String("shortcuts"));
 	for (int i=0;i<nshorts;i++) {
 		g.qs->setArrayIndex(i);
-		SAVELOAD(qmShortcuts[i], "keys");
-		SAVELOAD(qmShortcutSuppress[i], "suppress");
+		Shortcut s;
+
+		s.iIndex = -2;
+
+		SAVELOAD(s.iIndex, "index");
+		SAVELOAD(s.qlButtons, "keys");
+		SAVELOAD(s.bSuppress, "suppress");
+		if (s.iIndex >= -1)
+			qlShortcuts << s;
 	}
 	g.qs->endArray();
 
@@ -303,6 +317,13 @@ void Settings::load() {
 	}
 	g.qs->endArray();
 
+	g.qs->beginReadArray(QLatin1String("messagesounds"));
+	for (QMap<int, QString>::const_iterator it = qmMessageSounds.constBegin(); it != qmMessageSounds.constEnd(); ++it) {
+		g.qs->setArrayIndex(it.key());
+		SAVELOAD(qmMessageSounds[it.key()], "logsound");
+	}
+	g.qs->endArray();
+
 	g.qs->beginGroup(QLatin1String("lcd/devices"));
 	foreach(const QString &d, g.qs->childKeys()) {
 		qmLCDDevices.insert(d, g.qs->value(d, true).toBool());
@@ -334,7 +355,7 @@ void Settings::save() {
 	SAVELOAD(iIdleTime, "audio/idletime");
 	SAVELOAD(fAudioMinDistance, "audio/mindistance");
 	SAVELOAD(fAudioMaxDistance, "audio/maxdistance");
-	SAVELOAD(fAudioRollOff, "audio/rolloff");
+	SAVELOAD(fAudioMaxDistVolume, "audio/maxdistancevolume");
 	SAVELOAD(fAudioBloom, "audio/bloom");
 	SAVELOAD(bEcho, "audio/echo");
 	SAVELOAD(bPositionalAudio, "audio/positional");
@@ -426,10 +447,12 @@ void Settings::save() {
 	SAVELOAD(iLCDPlayerViewSplitterWidth, "lcd/playerview/splitterwidth");
 
 	g.qs->beginWriteArray(QLatin1String("shortcuts"));
-	for (ShortcutMap::const_iterator it = qmShortcuts.constBegin(); it != qmShortcuts.constEnd(); ++it) {
-		g.qs->setArrayIndex(it.key());
-		SAVELOAD(qmShortcuts[it.key()], "keys");
-		SAVELOAD(qmShortcutSuppress[it.key()], "suppress");
+	int idx = 0;
+	foreach(const Shortcut &s, qlShortcuts) {
+		g.qs->setArrayIndex(idx++);
+		g.qs->setValue(QLatin1String("index"), s.iIndex);
+		g.qs->setValue(QLatin1String("keys"), s.qlButtons);
+		g.qs->setValue(QLatin1String("suppress"), s.bSuppress);
 	}
 	g.qs->endArray();
 
@@ -440,6 +463,13 @@ void Settings::save() {
 	}
 	g.qs->endArray();
 
+	g.qs->beginWriteArray(QLatin1String("messagesounds"));
+	for (QMap<int, QString>::const_iterator it = qmMessageSounds.constBegin(); it != qmMessageSounds.constEnd(); ++it) {
+		g.qs->setArrayIndex(it.key());
+		SAVELOAD(qmMessageSounds[it.key()], "logsound");
+	}
+	g.qs->endArray();
+
 	g.qs->beginGroup(QLatin1String("lcd/devices"));
 	foreach(const QString &d, qmLCDDevices.keys()) {
 		bool v = qmLCDDevices.value(d);
