@@ -40,6 +40,13 @@
 // latter lives in a separate thread and so cannot touch the
 // GUI.
 
+struct Shortcut {
+	int iIndex;
+	QList<QVariant> qlButtons;
+	bool bSuppress;
+	bool operator <(const Shortcut &) const;
+};
+
 struct Settings {
 	enum AudioTransmit { Continous, VAD, PushToTalk };
 	enum VADSource { Amplitude, SignalToNoise };
@@ -60,7 +67,7 @@ struct Settings {
 	int iTTSVolume, iTTSThreshold;
 	int iQuality, iMinLoudness, iVoiceHold, iJitterBufferSize;
 	int iNoiseSuppress;
-	int iIdleTime;
+	unsigned int iIdleTime;
 	VADSource vsVAD;
 	float fVADmin, fVADmax;
 	int iFramesPerPacket;
@@ -84,7 +91,7 @@ struct Settings {
 	bool bEcho;
 	bool bPositionalAudio;
 	bool bPositionalHeadphone;
-	float fAudioMinDistance, fAudioMaxDistance, fAudioRollOff, fAudioBloom;
+	float fAudioMinDistance, fAudioMaxDistance, fAudioMaxDistVolume, fAudioBloom;
 
 	bool bOverlayEnable;
 	bool bOverlayUserTextures;
@@ -105,11 +112,10 @@ struct Settings {
 	int iLCDPlayerViewSplitterWidth;
 	QMap<QString, bool> qmLCDDevices;
 
-	typedef QMap<int, QList<QVariant > > ShortcutMap;
-	ShortcutMap qmShortcuts;
-	QMap<int, bool> qmShortcutSuppress;
+	QList<Shortcut> qlShortcuts;
 
-	enum MessageLog { LogConsole = 0x01, LogTTS = 0x02};
+	enum MessageLog { LogConsole = 0x01, LogTTS = 0x02, LogBalloon = 0x04, LogSoundfile = 0x08};
+	QMap<int, QString> qmMessageSounds;
 	QMap<int, quint32> qmMessages;
 
 	QString qsLanguage;
