@@ -64,7 +64,7 @@ int main(int argc, char **argv) {
 	os_init();
 #endif
 
-	QT_REQUIRE_VERSION(argc, argv, "4.3.0");
+	QT_REQUIRE_VERSION(argc, argv, "4.4.1");
 
 	// Initialize application object.
 	QApplication a(argc, argv);
@@ -73,14 +73,15 @@ int main(int argc, char **argv) {
 	a.setOrganizationDomain(QLatin1String("mumble.sourceforge.net"));
 	a.setQuitOnLastWindowClosed(false);
 
-#ifdef Q_OS_MAC
-	// Avoid auto-loading system Qt plugins. This could potentially make
-	// us load two different Qt libraries into our process. Not good.
-	a.setLibraryPaths(QStringList());
-#endif
-
 	Global::g_global_struct = new Global();
 
+	QUrl url;
+	if (a.arguments().count() > 1) {
+		QUrl u = QUrl::fromEncoded(a.arguments().last().toUtf8());
+		if (u.isValid() && (u.scheme() == QLatin1String("mumble")))
+			url = u;
+	}
+
 #ifdef USE_DBUS
 #ifdef Q_OS_WIN
 	// By default, windbus expects the path to dbus-daemon to be in PATH, and the path
@@ -100,12 +101,11 @@ int main(int argc, char **argv) {
 	}
 #endif
 
-	QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"));
+	QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 	QDBusMessage reply=qdbi.call(QLatin1String("focus"));
 	if (reply.type() == QDBusMessage::ReplyMessage) {
-		if (a.arguments().count() > 1) {
-			qdbi.call(QLatin1String("openUrl"), a.arguments().last());
-		}
+		if (url.isValid())
+			qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
 		return 0;
 	}
 #endif
@@ -184,6 +184,8 @@ int main(int argc, char **argv) {
 
 	g.lcd = new LCD();
 
+	g.nam = new QNetworkAccessManager();
+
 	// Process any waiting events before initializing our MainWindow.
 	// The mumble:// URL support for Mac OS X happens through AppleEvents,
 	// so we need to loop a little before we begin.  If we were launched
@@ -229,8 +231,8 @@ int main(int argc, char **argv) {
 	if (g.s.bUpdateCheck)
 		new VersionCheck(true, g.mw);
 
-	if (a.arguments().count() > 1) {
-		g.mw->openUrl(QUrl::fromEncoded(a.arguments().last().toLatin1()));
+	if (url.isValid()) {
+		g.mw->openUrl(url);
 #ifdef Q_OS_MAC
 	} else if (os_url) {
 		g.mw->openUrl(QUrl::fromEncoded(QByteArray(os_url)));
@@ -247,15 +249,16 @@ int main(int argc, char **argv) {
 	g.ai.reset();
 
 	g.sh->disconnect();
-
-	// This causes QT to complain. Fatally. It's either a bug in
-	// my code or the QT code.
-	// delete g.sh;
+	delete g.sh;
 	delete g.mw;
 
+	delete g.nam;
+	delete g.lcd;
+
 	delete g.db;
 	delete g.l;
 
+
 	delete g.o;
 
 	delete g.qs;
