@@ -4,6 +4,8 @@
 #define QT_ASCII_CAST_WARNINGS
 #define QT_NO_CAST_TO_ASCII
 
+#define _WINSOCKAPI_
+
 #if defined(__INTEL_COMPILER)
 #define _MSC_EXTENSIONS
 #include <mathimf.h>
@@ -11,6 +13,13 @@
 #define BOOST_TYPEOF_SUPPRESS_UNNAMED_NAMESPACE
 #endif
 
+#ifdef __APPLE__
+#include <Carbon/Carbon.h>
+#include <CoreFoundation/CoreFoundation.h>
+#include <ApplicationServices/ApplicationServices.h>
+#undef check
+#endif
+
 #include <QtCore/QtCore>
 #include <QtGui/QtGui>
 #ifdef USE_DBUS
@@ -30,6 +39,8 @@
 #include <speex/speex_echo.h>
 #include <speex/speex_callbacks.h>
 #include <speex/speex_resampler.h>
+#include <speex/speex_header.h>
+#include <ogg/ogg.h>
 
 #include <boost/shared_ptr.hpp>
 #include <boost/weak_ptr.hpp>
@@ -46,6 +57,8 @@
 #ifdef Q_OS_WIN
 #include <windows.h>
 #include <shellapi.h>
+#include <winsock2.h>
+#include <qos2.h>
 #ifndef Q_CC_INTEL
 #include <math.h>
 #define lroundf(x) ( static_cast<long int>( (x) + ((x) >= 0.0f ? 0.5f : -0.5f) ) )
