@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -137,21 +137,21 @@ void os_init() {
 
 
 #ifdef QT_NO_DEBUG
-	errno_t res;
-	res=_wfopen_s(&fConsole, L"Console.txt", L"a+");
+	errno_t res = 0;
+	size_t reqSize, bSize;
+	_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
+	if (reqSize > 0) {
+		reqSize += strlen("/Mumble/Console.txt");
+		bSize = reqSize;
+
+		STACKVAR(wchar_t, buff, reqSize+1);
+
+		_wgetenv_s(&reqSize, buff, bSize, L"APPDATA");
+		wcscat_s(buff, bSize, L"/Mumble/Console.txt");
+		res = _wfopen_s(&fConsole, buff, L"a+");
+	}
 	if ((res != 0) || (! fConsole)) {
-		size_t reqSize, bSize;
-		_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
-		if (reqSize > 0) {
-			reqSize += strlen("/Mumble/Console.txt");
-			bSize = reqSize;
-
-			STACKVAR(wchar_t, buff, reqSize+1);
-
-			_wgetenv_s(&reqSize, buff, bSize, L"APPDATA");
-			wcscat_s(buff, bSize, L"/Mumble/Console.txt");
-			res = _wfopen_s(&fConsole, buff, L"a+");
-		}
+		res=_wfopen_s(&fConsole, L"Console.txt", L"a+");
 	}
 	if ((res == 0) && fConsole)
 		qInstallMsgHandler(mumbleMessageOutput);
