@@ -176,125 +176,72 @@ QDBusConnection MurmurDBus::qdbc(QLatin1String("mainbus"));
 
 MurmurDBus::MurmurDBus(Server *srv) : QDBusAbstractAdaptor(srv) {
 	server = srv;
+	srv->connectListener(this);
 }
 
-void MurmurDBus::setTempGroups(int playerid, Channel *cChannel, const QStringList &groups) {
-	if (! cChannel)
-		cChannel = server->qhChannels.value(0);
-
-	Group *g;
-	foreach(g, cChannel->qhGroups)
-		g->qsTemporary.remove(playerid);
-
-	QString gname;
-	foreach(gname, groups) {
-		g = cChannel->qhGroups.value(gname);
-		if (! g) {
-			g = new Group(cChannel, gname);
-		}
-		g->qsTemporary.insert(playerid);
-	}
-
-	Player *p = server->qhUsers.value(playerid);
-	if (p)
-		server->clearACLCache(p);
+void MurmurDBus::removeAuthenticator() {
+	server->disconnectAuthenticator(this);
+	qsAuthPath = QString();
+	qsAuthService = QString();
 }
 
 
-QString MurmurDBus::mapIdToName(int id) {
-	if (qsAuthPath.isEmpty())
-		return QString();
-
+void MurmurDBus::idToNameSlot(QString &name, int id) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<QString> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getUserName",id);
 	if (reply.isValid())
-		return reply.value();
+		name = reply.value();
 	else {
-		server->log("DBus Authenticator failed getUserName for %d: %s", id, qPrintable(reply.error().message()));
-		qsAuthPath = QString();
-		qsAuthService = QString();
-		return QString();
+		server->log(QString("DBus Authenticator failed getUserName for %1: %2").arg(id).arg(reply.error().message()));
+		removeAuthenticator();
 	}
 }
 
-QByteArray MurmurDBus::mapIdToTexture(int id) {
-	if (qsAuthPath.isEmpty())
-		return QByteArray();
-
+void MurmurDBus::idToTextureSlot(QByteArray &qba, int id) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<QByteArray> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getUserTexture",id);
 	if (reply.isValid()) {
-		return reply.value();
-	} else {
-		// This is NOT a critical failure, it's an OPTIONAL implementation.
-		return QByteArray();
+		qba = reply.value();
 	}
 }
 
-int MurmurDBus::mapNameToId(const QString &name) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::nameToIdSlot(int &id, const QString &name) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getUserId",name);
 	if (reply.isValid())
-		return reply.value();
+		id = reply.value();
 	else {
-		server->log("DBus Authenticator failed getUserId for %s: %s", qPrintable(name), qPrintable(reply.error().message()));
-		qsAuthPath = QString();
-		qsAuthService = QString();
-		return -2;
+		server->log(QString("DBus Authenticator failed getUserId for %1: %2").arg(name).arg(reply.error().message()));
+		removeAuthenticator();
 	}
 }
 
-int MurmurDBus::dbusRegisterPlayer(const QString &name) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::registerPlayerSlot(int &res, const QString &name) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "registerPlayer", name);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::dbusUnregisterPlayer(int id) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::unregisterPlayerSlot(int &res, int id) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "unregisterPlayer", id);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::dbusGetRegistration(int id, QString &name, QString &email) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::getRegistrationSlot(int &res, int id, QString &name, QString &email) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<RegisteredPlayer> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getRegistration", id);
 	if (reply.isValid()) {
 		const RegisteredPlayer &r = reply.value();
 		name = r.name;
 		email = r.email;
-		return 1;
-	} else {
-		return -2;
+		res = 1;
 	}
 }
 
-QMap<int, QPair<QString, QString> > MurmurDBus::dbusGetRegisteredPlayers(const QString &filter) {
-	QMap<int, QPair<QString, QString> > m;
-
-	if (qsAuthPath.isEmpty())
-		return m;
-
+void  MurmurDBus::getRegisteredPlayersSlot(const QString &filter, QMap<int, QPair<QString, QString> > &m) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<QList<RegisteredPlayer> > reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "getRegisteredPlayers", filter);
 	if (reply.isValid()) {
@@ -303,65 +250,37 @@ QMap<int, QPair<QString, QString> > MurmurDBus::dbusGetRegisteredPlayers(const Q
 			m.insert(p.id, QPair<QString, QString>(p.name, p.email));
 		}
 	}
-	return m;
 }
 
-int MurmurDBus::dbusSetName(int id, const QString &name) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::setNameSlot(int &res, int id, const QString &name) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setName",id, name);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::dbusSetEmail(int id, const QString &email) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::setEmailSlot(int &res, int id, const QString &email) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setEmail",id, email);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::dbusSetPW(int id, const QString &pw) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::setPwSlot(int &res, int id, const QString &pw) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setPW",id, pw);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::dbusSetTexture(int id, const QByteArray &texture) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::setTextureSlot(int &res, int id, const QByteArray &texture) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusReply<int> reply = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "setTexture",id, texture);
 	if (reply.isValid())
-		return reply.value();
-	else {
-		return -2;
-	}
+		res = reply.value();
 }
 
-int MurmurDBus::authenticate(QString &uname, const QString &pw) {
-	if (qsAuthPath.isEmpty())
-		return -2;
-
+void MurmurDBus::authenticateSlot(int &res, QString &uname, const QString &pw) {
 	QDBusInterface remoteApp(qsAuthService,qsAuthPath,QString(),qdbc);
 	QDBusMessage msg = remoteApp.call(bReentrant ? QDBus::BlockWithGui : QDBus::Block, "authenticate",uname,pw);
 	QDBusError err = msg;
@@ -379,20 +298,17 @@ int MurmurDBus::authenticate(QString &uname, const QString &pw) {
 			}
 		}
 		if (ok && (msg.arguments().count() >= 3)) {
-			setTempGroups(uid, NULL, msg.arguments().at(2).toStringList());
+			server->setTempGroups(uid, NULL, msg.arguments().at(2).toStringList());
 		}
 		if (ok) {
-			server->log("DBus Authenticate success for %s: %d", qPrintable(uname),uid);
-			return uid;
+			server->log(QString("DBus Authenticate success for %1: %2").arg(uname).arg(uid));
+			res = uid;
 		} else {
-			server->log("DBus Autenticator failed authenticate for %s: Invalid return type", qPrintable(uname));
-			return -2;
+			server->log(QString("DBus Autenticator failed authenticate for %1: Invalid return type").arg(uname));
 		}
 	} else {
-		server->log("DBus Authenticator failed authenticate for %s: %s", qPrintable(uname), qPrintable(err.message()));
-		qsAuthPath = QString();
-		qsAuthService = QString();
-		return -2;
+		server->log(QString("DBus Authenticator failed authenticate for %1: %2").arg(uname).arg(err.message()));
+		removeAuthenticator();
 	}
 }
 
@@ -517,7 +433,7 @@ void MurmurDBus::sendMessageChannel(int id, bool tree, const QString &text, cons
 void MurmurDBus::addChannel(const QString &name, int chanparent, const QDBusMessage &msg, int &newid) {
 	CHANNEL_SETUP_VAR(chanparent);
 
-	Channel *nc = server->addChannel(cChannel, name);
+	Channel *nc = server->addChannel(cChannel, name, QString());
 	server->updateChannel(nc);
 	newid = nc->iId;
 
@@ -886,19 +802,22 @@ void MurmurDBus::verifyPassword(int id, const QString &pw, const QDBusMessage &m
 }
 
 void MurmurDBus::setAuthenticator(const QDBusObjectPath &path, bool reentrant, const QDBusMessage &msg) {
+	if (! qsAuthPath.isNull() || ! qsAuthService.isNull())
+		removeAuthenticator();
+	server->connectAuthenticator(this);
 	qsAuthPath = path.path();
 	qsAuthService = msg.service();
 	bReentrant = reentrant;
-	server->log("DBus Authenticator set to %s %s (%d)",qPrintable(qsAuthService),qPrintable(qsAuthPath), reentrant);
+	server->log(QString("DBus Authenticator set to %1 %2 (%3)").arg(qsAuthService).arg(qsAuthPath).arg(reentrant));
 }
 
 void MurmurDBus::setTemporaryGroups(int channel, int playerid, const QStringList &groups, const QDBusMessage &msg) {
 	CHANNEL_SETUP_VAR(channel);
 
-	setTempGroups(playerid, cChannel, groups);
+	server->setTempGroups(playerid, cChannel, groups);
 }
 
-PlayerInfo::PlayerInfo(Player *p) {
+PlayerInfo::PlayerInfo(const Player *p) {
 	session = p->uiSession;
 	mute = p->bMute;
 	deaf = p->bDeaf;
@@ -908,16 +827,16 @@ PlayerInfo::PlayerInfo(Player *p) {
 	channel = p->cChannel->iId;
 }
 
-PlayerInfoExtended::PlayerInfoExtended(Player *p) : PlayerInfo(p) {
+PlayerInfoExtended::PlayerInfoExtended(const Player *p) : PlayerInfo(p) {
 	id = p->iId;
 	name = p->qsName;
 
-	const User *u = static_cast<User *>(p);
+	const User *u = static_cast<const User *>(p);
 	onlinesecs = u->bwr.onlineSeconds();
 	bytespersec = u->bwr.bandwidth();
 }
 
-ChannelInfo::ChannelInfo(Channel *c) {
+ChannelInfo::ChannelInfo(const Channel *c) {
 	id = c->iId;
 	name = c->qsName;
 	parent = c->cParent ? c->cParent->iId : -1;
@@ -925,7 +844,7 @@ ChannelInfo::ChannelInfo(Channel *c) {
 		links << chn->iId;
 }
 
-ACLInfo::ACLInfo(ChanACL *acl) {
+ACLInfo::ACLInfo(const ChanACL *acl) {
 	applyHere = acl->bApplyHere;
 	applySubs = acl->bApplySubs;
 	inherited = false;
@@ -935,7 +854,7 @@ ACLInfo::ACLInfo(ChanACL *acl) {
 	deny = acl->pDeny;
 }
 
-GroupInfo::GroupInfo(Group *g) {
+GroupInfo::GroupInfo(const Group *g) {
 	name = g->qsName;
 	inherit = g->bInherit;
 	inheritable = g->bInheritable;
@@ -960,36 +879,41 @@ LogEntry::LogEntry(const ServerDB::LogRecord &r) {
 	txt = r.second;
 }
 
-void MurmurDBus::playerStateChanged(Player *p) {
+void MurmurDBus::playerStateChanged(const Player *p) {
 	emit playerStateChanged(PlayerInfo(p));
 }
 
-void MurmurDBus::playerConnected(Player *p) {
+void MurmurDBus::playerConnected(const Player *p) {
 	emit playerConnected(PlayerInfo(p));
 }
 
-void MurmurDBus::playerDisconnected(Player *p) {
+void MurmurDBus::playerDisconnected(const Player *p) {
 	emit playerDisconnected(PlayerInfo(p));
 }
 
-void MurmurDBus::channelStateChanged(Channel *c) {
+void MurmurDBus::channelStateChanged(const Channel *c) {
 	emit channelStateChanged(ChannelInfo(c));
 }
 
-void MurmurDBus::channelCreated(Channel *c) {
+void MurmurDBus::channelCreated(const Channel *c) {
 	emit channelCreated(ChannelInfo(c));
 }
 
-void MurmurDBus::channelRemoved(Channel *c) {
+void MurmurDBus::channelRemoved(const Channel *c) {
 	emit channelRemoved(ChannelInfo(c));
 }
 
 
 MetaDBus::MetaDBus(Meta *m) : QDBusAbstractAdaptor(m) {
 	meta = m;
+	m->connectListener(this);
 }
 
 void MetaDBus::started(Server *s) {
+	new MurmurDBus(s);
+	if (MurmurDBus::qdbc.isConnected())
+		MurmurDBus::qdbc.registerObject(QString::fromLatin1("/%1").arg(s->iServerNum), s);
+
 	emit started(s->iServerNum);
 }
 
@@ -1092,26 +1016,6 @@ void MetaDBus::setSuperUserPassword(int server_id, const QString &pw, const QDBu
 	}
 }
 
-extern QFile *qfLog;
-
-void MetaDBus::rotateLogs(const QDBusMessage &msg) {
-	if (! qfLog || ! qfLog->isOpen()) {
-		MurmurDBus::qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.nolog", "Logfile not in use"));
-		return;
-	}
-	qWarning("Logfile rotation requested from D-Bus, will reopen %s", qPrintable(Meta::mp.qsLogfile));
-	qfLog->close();
-	qfLog->setFileName(Meta::mp.qsLogfile);
-	if (! qfLog->open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
-		delete qfLog;
-		qfLog = NULL;
-		MurmurDBus::qdbc.send(msg.createErrorReply("net.sourceforge.mumble.Error.logfail", "Failed to reopen log"));
-	} else {
-		qfLog->setTextModeEnabled(true);
-		qWarning("Log rotated successfully");
-	}
-}
-
 void MetaDBus::quit() {
 	qCritical("Quit requested from D-Bus");
 	QCoreApplication::instance()->quit();
