@@ -55,7 +55,7 @@
 #define PERM_DENIED(who, where, what) \
 	mpd.qsReason = QString("%1 not allowed to %2 in %3").arg(who->qsName).arg(ChanACL::permName(what)).arg(where->qsName); \
 	sendMessage(cCon, &mpd); \
-	log(uSource, "%s", qPrintable(mpd.qsReason))
+	log(uSource, mpd.qsReason)
 #define PERM_DENIED_TEXT(text) \
 	mpd.qsReason = text; \
 	sendMessage(cCon, &mpd)
@@ -128,7 +128,7 @@ void Server::msgServerAuthenticate(Connection *cCon, MessageServerAuthenticate *
 	}
 
 	if (! ok) {
-		log(uSource, "Rejected connection: %s", qPrintable(msr.qsReason));
+		log(uSource, QString("Rejected connection: %1").arg(msr.qsReason));
 		sendMessage(cCon, &msr);
 		cCon->disconnectSocket();
 		return;
@@ -178,6 +178,13 @@ void Server::msgServerAuthenticate(Connection *cCon, MessageServerAuthenticate *
 
 		sendMessage(cCon, &mca);
 
+		MessageChannelDescUpdate mcdu;
+
+		mcdu.iId = c->iId;
+		mcdu.qsDesc = c->qsDesc;
+
+		sendMessage(static_cast<User *>(p), &mcdu);
+
 		foreach(c, c->qlChannels)
 			q.enqueue(c);
 	}
@@ -250,8 +257,13 @@ void Server::msgServerAuthenticate(Connection *cCon, MessageServerAuthenticate *
 	sendMessage(cCon, &mssMsg);
 	log(uSource, "Authenticated");
 
-	dbus->playerConnected(uSource);
+	emit playerConnected(uSource);
 	playerEnterChannel(uSource, lc, false);
+
+	while (lc && lc->qsDesc.isEmpty())
+		lc = lc->cParent;
+	if (lc)
+		sendChannelDescription(uSource, lc);
 }
 
 void Server::msgServerBanList(Connection *cCon, MessageServerBanList *msg) {
@@ -329,9 +341,8 @@ void Server::msgPlayerMute(Connection *cCon, MessagePlayerMute *msg) {
 		pDstUser->bDeaf = false;
 	}
 
-	dbus->playerStateChanged(pDstUser);
-
-	log(uSource, "Muted %s (%d)", qPrintable(pDstUser->qsName), msg->bMute);
+	log(uSource, QString("Muted %1 (%2)").arg(*pDstUser).arg(msg->bMute));
+	emit playerStateChanged(pDstUser);
 }
 
 void Server::msgPlayerDeaf(Connection *cCon, MessagePlayerDeaf *msg) {
@@ -353,9 +364,9 @@ void Server::msgPlayerDeaf(Connection *cCon, MessagePlayerDeaf *msg) {
 		pDstUser->bMute = true;
 	}
 
-	dbus->playerStateChanged(pDstUser);
+	log(uSource, QString("Deafened %1 (%2)").arg(*pDstUser).arg(msg->bDeaf));
 
-	log(uSource, "Deafened %s (%d)", qPrintable(pDstUser->qsName),msg->bDeaf);
+	emit playerStateChanged(pDstUser);
 }
 
 void Server::msgPlayerKick(Connection *cCon, MessagePlayerKick *msg) {
@@ -368,7 +379,7 @@ void Server::msgPlayerKick(Connection *cCon, MessagePlayerKick *msg) {
 	}
 
 	sendAll(msg);
-	log(uSource, "Kicked %s (%s)", qPrintable(pDstUser->qsName), qPrintable(msg->qsReason));
+	log(uSource, QString("Kicked %1 (%2)").arg(*pDstUser).arg(msg->qsReason));
 	pDstUser->disconnectSocket();
 }
 
@@ -382,7 +393,7 @@ void Server::msgPlayerBan(Connection *cCon, MessagePlayerBan *msg) {
 	}
 
 	sendAll(msg);
-	log(uSource, "Kickbanned %s (%s)", qPrintable(pDstUser->qsName), qPrintable(msg->qsReason));
+	log(uSource, QString("Kickbanned %1 (%2)").arg(*pDstUser).arg(msg->qsReason));
 
 	QHostAddress adr = pDstUser->peerAddress();
 	quint32 base = adr.toIPv4Address();
@@ -398,7 +409,8 @@ void Server::msgPlayerSelfMuteDeaf(Connection *cCon, MessagePlayerSelfMuteDeaf *
 	uSource->bSelfMute = msg->bMute;
 	uSource->bSelfDeaf = msg->bDeaf;
 	sendAll(msg);
-	dbus->playerStateChanged(uSource);
+
+	emit playerStateChanged(uSource);
 }
 
 void Server::msgPlayerMove(Connection *cCon, MessagePlayerMove *msg) {
@@ -421,7 +433,7 @@ void Server::msgPlayerMove(Connection *cCon, MessagePlayerMove *msg) {
 
 	sendAll(msg);
 	playerEnterChannel(pDstUser, c);
-	log(uSource, "Moved %s to %s (%d)",qPrintable(pDstUser->qsName), qPrintable(c->qsName), c->iId);
+	log(uSource, QString("Moved %1 to %2").arg(*pDstUser).arg(*c));
 }
 
 void Server::msgChannelAdd(Connection *cCon, MessageChannelAdd *msg) {
@@ -454,18 +466,18 @@ void Server::msgChannelAdd(Connection *cCon, MessageChannelAdd *msg) {
 		}
 	}
 
-	Channel *c = addChannel(p, msg->qsName);
+	Channel *c = addChannel(p, msg->qsName, QString());
 	if (uSource->iId >= 0) {
 		Group *g = new Group(c, "admin");
 		g->qsAdd << uSource->iId;
 	}
 	updateChannel(c);
 
-	dbus->channelCreated(c);
-
 	msg->iId = c->iId;
 	sendAll(msg);
-	log(uSource, "Added channel %s (%s)", qPrintable(c->qsName), qPrintable(p->qsName));
+	log(uSource, QString("Added channel %1 under %2").arg(*c).arg(*p));
+
+	emit channelCreated(c);
 }
 
 void Server::msgChannelRemove(Connection *cCon, MessageChannelRemove *msg) {
@@ -480,7 +492,7 @@ void Server::msgChannelRemove(Connection *cCon, MessageChannelRemove *msg) {
 		return;
 	}
 
-	log(uSource, "Removed channel %s",qPrintable(c->qsName));
+	log(uSource, QString("Removed channel %1").arg(*c));
 
 	removeChannel(c, uSource);
 }
@@ -515,11 +527,31 @@ void Server::msgChannelRename(Connection *cCon, MessageChannelRename *msg) {
 		}
 	}
 
-	log(uSource, "Renamed channel %s to %s", qPrintable(c->qsName), qPrintable(msg->qsName));
+	log(uSource, QString("Renamed channel %1 to %2").arg(*c).arg(msg->qsName));
 	c->qsName = msg->qsName;
 	updateChannel(c);
-	dbus->channelStateChanged(c);
 	sendAll(msg);
+
+	emit channelStateChanged(c);
+}
+
+void Server::msgChannelDescUpdate(Connection *cCon, MessageChannelDescUpdate *msg) {
+	MSG_SETUP(Player::Authenticated);
+	Channel *c = qhChannels.value(msg->iId);
+
+	if (!c)
+		return;
+
+	if (! hasPermission(uSource, c, ChanACL::Write)) {
+		PERM_DENIED(uSource, c, ChanACL::Write);
+		return;
+	}
+
+	log(uSource, QString("Changed description for channel %1 to: %2").arg(*c).arg(msg->qsDesc));
+	c->qsDesc = msg->qsDesc;
+	updateChannel(c);
+	sendAll(msg);
+	sendChannelDescriptionUpdate(c);
 }
 
 void Server::msgChannelMove(Connection *cCon, MessageChannelMove *msg) {
@@ -558,13 +590,14 @@ void Server::msgChannelMove(Connection *cCon, MessageChannelMove *msg) {
 		p = p->cParent;
 	}
 
-	log(uSource, "Moved channel %s from %s to %s", qPrintable(c->qsName), qPrintable(c->cParent->qsName), qPrintable(np->qsName));
+	log(uSource, QString("Moved channel %1 from %2 to %3").arg(*c).arg(* c->cParent).arg(*np));
 
 	c->cParent->removeChannel(c);
 	np->addChannel(c);
 	updateChannel(c);
-	dbus->channelStateChanged(c);
 	sendAll(msg);
+
+	emit channelStateChanged(c);
 }
 
 void Server::msgChannelLink(Connection *cCon, MessageChannelLink *msg) {
@@ -593,25 +626,25 @@ void Server::msgChannelLink(Connection *cCon, MessageChannelLink *msg) {
 			}
 			c->link(l);
 			addLink(c, l);
-			dbus->channelStateChanged(c);
-			log(uSource, "Linked channel %s to %s", qPrintable(c->qsName),qPrintable(l->qsName));
+			log(uSource, QString("Linked channel %1 to %2").arg(*c).arg(*l));
+			emit channelStateChanged(c);
 			break;
 		case MessageChannelLink::Unlink:
 			if (!l)
 				return;
 			c->unlink(l);
 			removeLink(c, l);
-			dbus->channelStateChanged(c);
-			log(uSource, "Unlinked channel %s from %s", qPrintable(c->qsName), qPrintable(l->qsName));
+			log(uSource, QString("Unlinked channel %1 to %2").arg(*c).arg(*l));
+			emit channelStateChanged(c);
 			break;
 		case MessageChannelLink::UnlinkAll:
 			if (msg->qlTargets.count() > 0)
 				return;
 			c->unlink(NULL);
 			removeLink(c, NULL);
-			dbus->channelStateChanged(c);
-			log(uSource, "Unlinked all from channel %s", qPrintable(c->qsName));
+			log(uSource, QString("Unlinked all from channel %1").arg(*c));
 			sendAll(msg);
+			emit channelStateChanged(c);
 			return;
 		case MessageChannelLink::PushLink:
 			if (msg->qlTargets.count() <= 0)
@@ -711,7 +744,7 @@ void Server::msgEditACL(Connection *cCon, MessageEditACL *msg) {
 	if (!c)
 		return;
 
-	if (! hasPermission(uSource, c, ChanACL::Write)) {
+	if (! hasPermission(uSource, c, ChanACL::Write) && !(c->cParent && hasPermission(uSource, c->cParent, ChanACL::Write))) {
 		PERM_DENIED(uSource, c, ChanACL::Write);
 		return;
 	}
@@ -827,7 +860,7 @@ void Server::msgEditACL(Connection *cCon, MessageEditACL *msg) {
 		}
 
 		updateChannel(c);
-		log(uSource, "Updated ACL in channel %s[%d]", qPrintable(c->qsName), c->iId);
+		log(uSource, QString("Updated ACL in channel %1").arg(*c));
 	}
 }
 
@@ -926,3 +959,16 @@ void Server::msgCryptSync(Connection *cCon, MessageCryptSync *msg) {
 		cCon->disconnectSocket();
 	}
 }
+
+void Server::msgContextAddAction(Connection *cCon, MessageContextAddAction *) {
+	cCon->disconnectSocket();
+}
+
+void Server::msgContextAction(Connection *cCon, MessageContextAction *msg) {
+	MSG_SETUP(Player::Authenticated);
+	if ((msg->uiVictim > 0) && ! qhUsers.contains(msg->uiVictim))
+		return;
+	if ((msg->iChannel >= 0) && ! qhChannels.contains(msg->iChannel))
+		return;
+	emit contextAction(uSource, msg->qsAction, msg->uiVictim, msg->iChannel);
+}
