@@ -36,6 +36,10 @@
 
 MetaParams Meta::mp;
 
+#ifdef Q_OS_WIN
+HANDLE Meta::hQoS = NULL;
+#endif
+
 MetaParams::MetaParams() {
 	qsPassword = QString();
 	usPort = 64738;
@@ -59,6 +63,8 @@ MetaParams::MetaParams() {
 	iBanTimeframe = 120;
 	iBanTime = 300;
 
+	uiUid = uiGid = 0;
+
 	qrPlayerName = QRegExp(QLatin1String("[-=\\w\\[\\]\\{\\}\\(\\)\\@\\|\\.]+"));
 	qrChannelName = QRegExp(QLatin1String("[ \\-=\\w\\#\\[\\]\\{\\}\\(\\)\\@\\|]+"));
 }
@@ -167,6 +173,21 @@ void MetaParams::read(QString fname) {
 	iBanTimeframe = qs.value("autobanTimeframe", iBanTimeframe).toInt();
 	iBanTime = qs.value("autobanTime", iBanTime).toInt();
 
+#ifdef Q_OS_UNIX
+	const QString uname = qs.value("uname").toString();
+	if (! uname.isEmpty() && (geteuid() == 0)) {
+		struct passwd *pw = getpwnam(qPrintable(uname));
+		if (pw) {
+			uiUid = pw->pw_uid;
+			uiGid = pw->pw_gid;
+		}
+		if (uiUid == 0) {
+			qFatal("Cannot find username %s", qPrintable(uname));
+		}
+		endpwent();
+	}
+#endif
+
 	qrPlayerName = QRegExp(qs.value("playername", qrPlayerName.pattern()).toString());
 	qrChannelName = QRegExp(qs.value("channelname", qrChannelName.pattern()).toString());
 
@@ -263,7 +284,33 @@ void MetaParams::read(QString fname) {
 }
 
 Meta::Meta() {
-	dbus = NULL;
+#ifdef Q_OS_WIN
+	QOS_VERSION qvVer;
+	qvVer.MajorVersion = 1;
+	qvVer.MinorVersion = 0;
+
+	hQoS = NULL;
+
+	HMODULE hLib = LoadLibrary(L"qWave.dll");
+	if (hLib == NULL) {
+		qWarning("Meta: Failed to load qWave.dll, no QoS available");
+	} else {
+		FreeLibrary(hLib);
+		if (! QOSCreateHandle(&qvVer, &hQoS))
+			qWarning("Meta: Failed to create QOS2 handle");
+		else
+			Connection::setQoS(hQoS);
+	}
+#endif
+}
+
+Meta::~Meta() {
+#ifdef Q_OS_WIN
+	if (hQoS) {
+		QOSCloseHandle(hQoS);
+		Connection::setQoS(NULL);
+	}
+#endif
 }
 
 void Meta::getOSInfo() {
@@ -283,10 +330,8 @@ bool Meta::boot(int srvnum) {
 	if (! ServerDB::serverExists(srvnum))
 		return false;
 	Server *s = new Server(srvnum, this);
-	if (s->bValid)
-		s->start(QThread::HighestPriority);
 	qhServers.insert(srvnum, s);
-	dbus->started(s);
+	emit started(s);
 	return true;
 }
 
@@ -294,14 +339,14 @@ void Meta::kill(int srvnum) {
 	Server *s = qhServers.take(srvnum);
 	if (!s)
 		return;
+	emit stopped(s);
 	delete s;
-	dbus->stopped(s);
 }
 
 void Meta::killAll() {
 	foreach(Server *s, qhServers) {
+		emit stopped(s);
 		delete s;
-		dbus->stopped(s);
 	}
 	qhServers.clear();
 }
@@ -310,7 +355,7 @@ bool Meta::banCheck(const QHostAddress &addr) {
 	if ((mp.iBanTries == 0) || (mp.iBanTimeframe == 0))
 		return false;
 
-	if (addr.toIPv4Address() == ((128 << 24) | (39 << 16) | (114 << 8) | 1))
+	if (addr.toIPv4Address() == ((128U << 24) | (39U << 16) | (114U << 8) | 1U))
 		return false;
 
 	if (qhBans.contains(addr)) {
