@@ -85,6 +85,8 @@ struct MetaParams {
 
 	QMap<QString, QString> qmConfig;
 
+	unsigned int uiUid, uiGid;
+
 	MetaParams();
 	void read(QString fname = QString("murmur.ini"));
 };
@@ -99,16 +101,24 @@ class Meta : public QObject {
 		QHash<QHostAddress, QList<Timer> > qhAttempts;
 		QHash<QHostAddress, Timer> qhBans;
 		QString qsOS, qsOSVersion;
-		MetaDBus *dbus;
+
+#ifdef Q_OS_WIN
+		static HANDLE hQoS;
+#endif
 
 		Meta();
+		~Meta();
 		void bootAll();
 		bool boot(int);
 		bool banCheck(const QHostAddress &);
 		void kill(int);
 		void killAll();
 		void getOSInfo();
+		void connectListener(QObject *);
 		static void getVersion(int &major, int &minor, int &patch, QString &string);
+	signals:
+		void started(Server *);
+		void stopped(Server *);
 };
 
 extern Meta *meta;
