@@ -20,6 +20,11 @@ class ServerI : virtual public Server {
 		virtual void delete_async(const ::Murmur::AMD_Server_deletePtr&,
 		                          const Ice::Current&);
 
+		virtual void addCallback_async(const ::Murmur::AMD_Server_addCallbackPtr&, const ::Murmur::ServerCallbackPrx&, const ::Ice::Current&);
+		virtual void removeCallback_async(const ::Murmur::AMD_Server_removeCallbackPtr&, const ::Murmur::ServerCallbackPrx&, const ::Ice::Current&);
+
+		virtual void setAuthenticator_async(const ::Murmur::AMD_Server_setAuthenticatorPtr&, const ::Murmur::ServerAuthenticatorPrx&, const ::Ice::Current&);
+
 		virtual void id_async(const ::Murmur::AMD_Server_idPtr&,
 		                      const Ice::Current&);
 
@@ -35,9 +40,9 @@ class ServerI : virtual public Server {
 		                           const ::std::string&,
 		                           const Ice::Current&);
 
-		virtual void setSuperuserPasssword_async(const ::Murmur::AMD_Server_setSuperuserPassswordPtr&,
-		        const ::std::string&,
-		        const Ice::Current&);
+		virtual void setSuperuserPassword_async(const ::Murmur::AMD_Server_setSuperuserPasswordPtr&,
+		                                        const ::std::string&,
+		                                        const Ice::Current&);
 
 		virtual void getLog_async(const ::Murmur::AMD_Server_getLogPtr&,
 		                          ::Ice::Int,
@@ -70,6 +75,11 @@ class ServerI : virtual public Server {
 		                               const ::std::string&,
 		                               const Ice::Current&);
 
+		virtual void hasPermission_async(const ::Murmur::AMD_Server_hasPermissionPtr&, ::Ice::Int, ::Ice::Int, ::Ice::Int, const ::Ice::Current&);
+
+		virtual void addContextCallback_async(const ::Murmur::AMD_Server_addContextCallbackPtr&, ::Ice::Int, const ::std::string&, const ::std::string&, const ::Murmur::ServerContextCallbackPrx&, int, const ::Ice::Current&);
+		virtual void removeContextCallback_async(const ::Murmur::AMD_Server_removeContextCallbackPtr&, const ::Murmur::ServerContextCallbackPrx&, const ::Ice::Current&);
+
 		virtual void getState_async(const ::Murmur::AMD_Server_getStatePtr&,
 		                            ::Ice::Int,
 		                            const Ice::Current&);
@@ -178,6 +188,10 @@ class MetaI : virtual public Meta {
 
 		virtual void getVersion_async(const ::Murmur::AMD_Meta_getVersionPtr&,
 		                              const Ice::Current&);
+
+
+		virtual void addCallback_async(const ::Murmur::AMD_Meta_addCallbackPtr&, const ::Murmur::MetaCallbackPrx&, const ::Ice::Current& = ::Ice::Current());
+		virtual void removeCallback_async(const ::Murmur::AMD_Meta_removeCallbackPtr&, const ::Murmur::MetaCallbackPrx&, const ::Ice::Current& = ::Ice::Current());
 };
 
 }
