@@ -87,10 +87,10 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 	connect(qtsServer, SIGNAL(newConnection()), this, SLOT(newClient()), Qt::QueuedConnection);
 
 	if (! qtsServer->listen(qhaBind, usPort)) {
-		log("Server: TCP Listen on port %d failed",usPort);
+		log(QString("Server: TCP Listen on port %1 failed").arg(usPort));
 		bValid = false;
 	} else {
-		log("Server listening on port %d",usPort);
+		log(QString("Server listening on port %1").arg(usPort));
 	}
 
 	sUdpSocket = INVALID_SOCKET;
@@ -106,7 +106,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 		DWORD dwBytesReturned = 0;
 		BOOL bNewBehaviour = FALSE;
 		if (WSAIoctl(sUdpSocket, SIO_UDP_CONNRESET, &bNewBehaviour, sizeof(bNewBehaviour), NULL, 0, &dwBytesReturned, NULL, NULL) == SOCKET_ERROR) {
-			log("Failed to set SIO_UDP_CONNRESET: %d", WSAGetLastError());
+			log(QString("Failed to set SIO_UDP_CONNRESET: %1").arg(WSAGetLastError()));
 		}
 #endif
 		if (sUdpSocket == INVALID_SOCKET) {
@@ -119,12 +119,15 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 			addr.sin_port = htons(usPort);
 			addr.sin_addr.s_addr = htonl(qhaBind.toIPv4Address());
 			if (::bind(sUdpSocket, reinterpret_cast<sockaddr *>(&addr), sizeof(addr)) == SOCKET_ERROR) {
-				log("Failed to bind UDP Socket to port %d", usPort);
+				log(QString("Failed to bind UDP Socket to port %1").arg(usPort));
 			} else {
 #ifdef Q_OS_UNIX
-				int val = IPTOS_PREC_FLASHOVERRIDE | IPTOS_LOWDELAY | IPTOS_THROUGHPUT;
-				if (setsockopt(sUdpSocket, IPPROTO_IP, IP_TOS, &val, sizeof(val)))
-					log("Server: Failed to set TOS for UDP Socket");
+				int val = 0xe0;
+				if (setsockopt(sUdpSocket, IPPROTO_IP, IP_TOS, &val, sizeof(val))) {
+					val = 0x80;
+					if (setsockopt(sUdpSocket, IPPROTO_IP, IP_TOS, &val, sizeof(val)))
+						log("Server: Failed to set TOS for UDP Socket");
+				}
 #endif
 			}
 		}
@@ -138,7 +141,6 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 
 	qtTimeout = new QTimer(this);
 	connect(qtTimeout, SIGNAL(timeout()), this, SLOT(checkTimeout()));
-	qtTimeout->start(15500);
 
 	getBans();
 	readChannels();
@@ -147,26 +149,49 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 
 	if (bValid)
 		initRegister();
+}
 
-	dbus = new MurmurDBus(this);
-	if (MurmurDBus::qdbc.isConnected())
-		MurmurDBus::qdbc.registerObject(QString::fromLatin1("/%1").arg(iServerNum), this);
+void Server::startThread() {
+	if (! isRunning()) {
+		bRunning = true;
+		start(QThread::HighestPriority);
+#ifdef Q_OS_LINUX
+		// QThread::HighestPriority == Same as everything else...
+		int policy;
+		struct sched_param param;
+		if (pthread_getschedparam(pthread_self(), &policy, &param) == 0) {
+			if (policy == SCHED_OTHER) {
+				policy = SCHED_FIFO;
+				param.sched_priority = 1;
+				pthread_setschedparam(pthread_self(), policy, &param);
+			}
+		}
+#endif
+	}
+	if (! qtTimeout->isActive())
+		qtTimeout->start(15500);
+}
 
-	bRunning = true;
+void Server::stopThread() {
+	bRunning = false;
+	if (isRunning()) {
+		qrwlUsers.lockForWrite();
+		terminate();
+		wait();
+		qrwlUsers.unlock();
+	}
+	qtTimeout->stop();
 }
 
 Server::~Server() {
-	qrwlUsers.lockForWrite();
-	bRunning = false;
-	terminate();
-	wait();
-	qrwlUsers.unlock();
+	stopThread();
 	if (sUdpSocket != INVALID_SOCKET)
 #ifdef Q_OS_UNIX
 		close(sUdpSocket);
 #else
 		closesocket(sUdpSocket);
 #endif
+	clearACLCache();
 	log("Stopped");
 }
 
@@ -197,12 +222,12 @@ void Server::readParams() {
 				}
 			}
 			if ((qhaBind == QHostAddress::Any) || (qhaBind.isNull())) {
-				log("Lookup of bind hostname %s failed", qPrintable(qsHost));
+				log(QString("Lookup of bind hostname %1 failed").arg(qsHost));
 				qhaBind = Meta::mp.qhaBind;
 			}
 
 		}
-		log("Binding to address %s", qPrintable(qhaBind.toString()));
+		log(QString("Binding to address %1").arg(qhaBind.toString()));
 	}
 
 	qsPassword = getConf("password", qsPassword).toString();
@@ -245,6 +270,10 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		qsRegHost = !v.isNull() ? v : Meta::mp.qsRegHost;
 	else if (key == "registerurl")
 		qurlRegWeb = !v.isNull() ? v : Meta::mp.qurlRegWeb;
+	else if (key == "playername")
+		qrPlayerName=!v.isNull() ? QRegExp(v) : Meta::mp.qrPlayerName;
+	else if (key == "channelname")
+		qrChannelName=!v.isNull() ? QRegExp(v) : Meta::mp.qrChannelName;
 }
 
 
@@ -291,12 +320,12 @@ int BandwidthRecord::bandwidth() const {
 void Server::run() {
 	qint32 len;
 #if defined(__LP64__)
-	char encbuff[65536+8];
+	char encbuff[512+8];
 	char *encrypted = encbuff + 4;
 #else
-	char encrypted[65536];
+	char encrypted[512];
 #endif
-	char buffer[65536];
+	char buffer[512];
 
 	quint32 msgType = 0;
 	unsigned int uiSession = 0;
@@ -334,8 +363,11 @@ void Server::run() {
 #endif
 
 		fromlen = sizeof(from);
-		len=::recvfrom(sUdpSocket, encrypted, 65535, 0, reinterpret_cast<struct sockaddr *>(&from), &fromlen);
-
+#ifdef Q_OS_WIN
+		len=::recvfrom(sUdpSocket, encrypted, 512, 0, reinterpret_cast<struct sockaddr *>(&from), &fromlen);
+#else
+		len=::recvfrom(sUdpSocket, encrypted, 512, MSG_TRUNC, reinterpret_cast<struct sockaddr *>(&from), &fromlen);
+#endif
 		if (len == 0) {
 			break;
 		} else if (len == SOCKET_ERROR) {
@@ -343,6 +375,8 @@ void Server::run() {
 		} else if (len < 6) {
 			// 4 bytes crypt header + type + session
 			continue;
+		} else if (len > 512) {
+			continue;
 		}
 
 		QReadLocker rl(&qrwlUsers);
@@ -405,9 +439,12 @@ void Server::run() {
 }
 
 void Server::fakeUdpPacket(Message *msg, Connection *source) {
-	char buffer[65535];
-	PacketDataStream pds(buffer, 65535);
+	char buffer[512];
+	PacketDataStream pds(buffer, 512);
 	msg->messageToNetwork(pds);
+	if (! pds.isValid())
+		return;
+
 	pds.rewind();
 
 	quint32 msgType;
@@ -441,7 +478,18 @@ void Server::sendMessage(User *u, const char *data, int len, QByteArray &cache)
 		STACKVAR(char, buffer, len+4);
 #endif
 		u->csCrypt.encrypt(reinterpret_cast<const unsigned char *>(data), reinterpret_cast<unsigned char *>(buffer), len);
+#ifdef Q_OS_WIN
+		DWORD dwFlow = 0;
+		if (Meta::hQoS)
+			QOSAddSocketToFlow(Meta::hQoS, sUdpSocket, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), QOSTrafficTypeVoice, QOS_NON_ADAPTIVE_FLOW, &dwFlow);
 		::sendto(sUdpSocket, buffer, len+4, 0, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), sizeof(u->saiUdpAddress));
+		if (Meta::hQoS && dwFlow)
+			QOSRemoveSocketFromFlow(Meta::hQoS, 0, dwFlow, 0);
+#else
+		::sendto(sUdpSocket, buffer, len+4, 0, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), sizeof(u->saiUdpAddress));
+#endif
+
+
 	} else {
 		if (cache.isEmpty())
 			cache = QByteArray(data, len);
@@ -509,29 +557,14 @@ void Server::processMsg(PacketDataStream &pds, Connection *cCon) {
 	}
 }
 
-void Server::log(User *u, const char *format, ...) {
-	char buffer[4096];
-	va_list ap;
-	va_start(ap, format);
-	vsnprintf(buffer, 4096, format, ap);
-	va_end(ap);
-
-	char fin[4096];
-	snprintf(fin, 4096, "<%d:%s(%d)> %s", u->uiSession, qPrintable(u->qsName), u->iId, buffer);
-
-	dblog(fin);
-	qWarning("%d => %s", iServerNum, fin);
+void Server::log(User *u, const QString &str) {
+	QString msg = QString("<%1:%2(%3)> %4").arg(u->uiSession).arg(u->qsName).arg(u->iId).arg(str);
+	log(msg);
 }
 
-void Server::log(const char *format, ...) {
-	char buffer[4096];
-	va_list ap;
-	va_start(ap, format);
-	vsnprintf(buffer, 4096, format, ap);
-	va_end(ap);
-
-	dblog(buffer);
-	qWarning("%d => %s", iServerNum, buffer);
+void Server::log(const QString &msg) {
+	dblog(msg);
+	qWarning("%d => %s", iServerNum, msg.toUtf8().constData());
 }
 
 void Server::newClient() {
@@ -544,7 +577,7 @@ void Server::newClient() {
 		quint32 base = adr.toIPv4Address();
 
 		if (meta->banCheck(adr)) {
-			log("Ignoring connection: %s:%d (Global ban)",qPrintable(addressToString(sock->peerAddress())),sock->peerPort());
+			log(QString("Ignoring connection: %1:%2 (Global ban)").arg(addressToString(sock->peerAddress())).arg(sock->peerPort()));
 			sock->disconnectFromHost();
 			sock->deleteLater();
 			return;
@@ -556,7 +589,7 @@ void Server::newClient() {
 			int mask = 32 - ban.second;
 			mask = (1 << mask) - 1;
 			if ((base & ~mask) == (ban.first & ~mask)) {
-				log("Ignoring connection: %s:%d",qPrintable(addressToString(sock->peerAddress())),sock->peerPort());
+				log(QString("Ignoring connection: %1:%2 (Server ban)").arg(addressToString(sock->peerAddress())).arg(sock->peerPort()));
 				sock->disconnectFromHost();
 				sock->deleteLater();
 				return;
@@ -572,6 +605,9 @@ void Server::newClient() {
 			return;
 		}
 
+		if (qhUsers.isEmpty())
+			startThread();
+
 		User *u = new User(this, sock);
 		u->uiSession = qqIds.dequeue();
 
@@ -585,7 +621,9 @@ void Server::newClient() {
 		connect(u, SIGNAL(message(QByteArray &)), this, SLOT(message(QByteArray &)));
 		connect(u, SIGNAL(handleSslErrors(const QList<QSslError> &)), this, SLOT(sslError(const QList<QSslError> &)));
 
-		log(u, "New connection: %s:%d",qPrintable(addressToString(sock->peerAddress())),sock->peerPort());
+		log(u, QString("New connection: %1:%2").arg(addressToString(sock->peerAddress())).arg(sock->peerPort()));
+
+		u->setToS();
 
 		sock->startServerEncryption();
 	}
@@ -617,14 +655,14 @@ void Server::connectionClosed(QString reason) {
 		return;
 	User *u = static_cast<User *>(c);
 
-	log(u, "Connection closed: %s", qPrintable(reason));
+	log(u, QString("Connection closed: %1").arg(reason));
 
 	if (u->sState == Player::Authenticated) {
 		MessageServerLeave mslMsg;
 		mslMsg.uiSession=u->uiSession;
 		sendExcept(&mslMsg, c);
 
-		dbus->playerDisconnected(u);
+		emit playerDisconnected(u);
 	}
 
 	{
@@ -646,6 +684,9 @@ void Server::connectionClosed(QString reason) {
 		clearACLCache(u);
 
 	u->deleteLater();
+
+	if (qhUsers.isEmpty())
+		stopThread();
 }
 
 void Server::message(QByteArray &qbaMsg, Connection *cCon) {
@@ -709,6 +750,24 @@ void Server::sendExcept(Message *mMsg, Connection *cCon) {
 			u->sendMessage(mMsg);
 }
 
+void Server::sendChannelDescription(Player *p, const Channel *c) {
+	sendTextMessage(NULL, static_cast<User*>(p), false, QString("<u>%1:</u><br />%2").arg(c->qsName).arg(c->qsDesc));
+}
+
+void Server::sendChannelDescriptionUpdate(const Channel *changed, const Channel *current) {
+	if (current==NULL)
+		current=changed;
+
+	foreach(Player *siblingPlayer, current->qlPlayers) {
+		sendChannelDescription(siblingPlayer, changed);
+	}
+
+	foreach(Channel *siblingChannel, current->qlChannels) {
+		if (siblingChannel->qsDesc.isEmpty())
+			sendChannelDescriptionUpdate(changed, siblingChannel);
+	}
+}
+
 void Server::removeChannel(Channel *chan, Player *src, Channel *dest) {
 	Channel *c;
 	Player *p;
@@ -740,7 +799,7 @@ void Server::removeChannel(Channel *chan, Player *src, Channel *dest) {
 	sendAll(&mcr);
 
 	removeChannel(chan);
-	dbus->channelRemoved(chan);
+	emit channelRemoved(chan);
 
 	if (chan->cParent) {
 		QWriteLocker wl(&qrwlUsers);
@@ -756,6 +815,12 @@ void Server::playerEnterChannel(Player *p, Channel *c, bool quiet) {
 	if (quiet && (p->cChannel == c))
 		return;
 
+	// Get old and new channel of user
+	// Lookup first channel in list of parent channels of new and old one
+	// which has a channel description
+	const Channel *oldChannel=p->cChannel;
+	const Channel *newChannel=c;
+
 	{
 		QWriteLocker wl(&qrwlUsers);
 		c->addPlayer(p);
@@ -764,8 +829,19 @@ void Server::playerEnterChannel(Player *p, Channel *c, bool quiet) {
 	if (quiet)
 		return;
 
+	while (oldChannel && oldChannel->qsDesc.isEmpty())
+		oldChannel=oldChannel->cParent;
+	while (newChannel && newChannel->qsDesc.isEmpty())
+		newChannel=newChannel->cParent;
+
+	// Only send message with channel description if it changed
+	// That means the channel has its own description or it does not
+	// share the same first parent channel which has one with the channel
+	// the user came from
+	if (newChannel && newChannel!=oldChannel)
+		sendChannelDescription(p, newChannel);
+
 	setLastChannel(p);
-	dbus->playerStateChanged(p);
 
 	bool mayspeak = hasPermission(p, c, ChanACL::Speak);
 	bool sup = p->bSuppressed;
@@ -782,6 +858,7 @@ void Server::playerEnterChannel(Player *p, Channel *c, bool quiet) {
 			sendAll(&mpm);
 		}
 	}
+	emit playerStateChanged(p);
 }
 
 bool Server::hasPermission(Player *p, Channel *c, ChanACL::Perm perm) {
