@@ -100,20 +100,16 @@ class User : public Connection, public Player {
 		User(Server *parent, QSslSocket *socket);
 };
 
-class DBus;
-namespace Murmur {
-class ServerI;
-};
-
 class Server : public QThread, public MessageHandler {
-		friend class DBus;
-		friend class Murmur::ServerI;
 	private:
 		Q_OBJECT;
 		Q_DISABLE_COPY(Server);
 	protected:
 		bool bRunning;
 
+		void startThread();
+		void stopThread();
+
 		// Former ServerParams
 	public:
 		QHostAddress qhaBind;
@@ -187,8 +183,6 @@ class Server : public QThread, public MessageHandler {
 		QReadWriteLock qrwlUsers;
 		ChanACL::ACLCache acCache;
 		QMutex qmCache;
-		MurmurDBus *dbus;
-
 		QHash<int, QByteArray> qhUserTextureCache;
 		QHash<int, QString> qhUserNameCache;
 		QHash<QString, int> qhUserIDCache;
@@ -211,19 +205,15 @@ class Server : public QThread, public MessageHandler {
 		void sendAll(Message *);
 		void sendExcept(Message *, Connection *);
 		void sendMessage(Connection *, Message *);
+		void sendChannelDescription(Player *, const Channel *);
+		void sendChannelDescriptionUpdate(const Channel *changed, const Channel *current=NULL);
 
 		void setLiveConf(const QString &key, const QString &value);
 
 		QString addressToString(const QHostAddress &);
 
-#ifndef Q_OS_WIN
-		__attribute__((format(printf, 2, 3)))
-#endif
-		void log(const char *format, ...);
-#ifndef Q_OS_WIN
-		__attribute__((format(printf, 3, 4)))
-#endif
-		void log(User *u, const char *format, ...);
+		void log(const QString &);
+		void log(User *u, const QString &);
 
 		void removeChannel(Channel *c, Player *src, Channel *dest = NULL);
 		void playerEnterChannel(Player *u, Channel *c, bool quiet = false);
@@ -232,13 +222,33 @@ class Server : public QThread, public MessageHandler {
 		~Server();
 
 		// RPC functions. Implementation in RPC.cpp
+		void connectAuthenticator(QObject *p);
+		void disconnectAuthenticator(QObject *p);
+		void connectListener(QObject *p);
+		void disconnectListener(QObject *p);
+		void setTempGroups(const int playerid, Channel *cChannel, const QStringList &groups);
 	signals:
+		void registerPlayerSig(int &, const QString &);
+		void unregisterPlayerSig(int &, int);
+		void getRegisteredPlayersSig(const QString &, QMap<int, QPair<QString, QString> > &);
+		void getRegistrationSig(int &, int, QString &, QString &);
+		void authenticateSig(int &, QString &, const QString &);
+		void setPwSig(int &, int, const QString &);
+		void setEmailSig(int &, int, const QString &);
+		void setNameSig(int &, int, const QString &);
+		void setTextureSig(int &, int, const QByteArray &);
+		void idToNameSig(QString &, int);
+		void nameToIdSig(int &, const QString &);
+		void idToTextureSig(QByteArray &, int);
+
 		void playerStateChanged(const Player *);
 		void playerConnected(const Player *);
 		void playerDisconnected(const Player *);
 		void channelStateChanged(const Channel *);
 		void channelCreated(const Channel *);
 		void channelRemoved(const Channel *);
+
+		void contextAction(const Player *, const QString &, unsigned int, int);
 	public:
 		void setPlayerState(Player *p, Channel *parent, bool mute, bool deaf, bool suppressed);
 		bool setChannelState(Channel *c, Channel *parent, const QString &qsName, const QSet<Channel *> &links);
@@ -248,7 +258,7 @@ class Server : public QThread, public MessageHandler {
 		void initialize();
 		typedef QPair<quint32, int> qpBan;
 		int authenticate(QString &name, const QString &pw);
-		Channel *addChannel(Channel *c, const QString &name);
+		Channel *addChannel(Channel *c, const QString &name, const QString &desc = QString());
 		void removeChannel(const Channel *c);
 		void readChannels(Channel *p = NULL);
 		void readLinks();
@@ -275,7 +285,7 @@ class Server : public QThread, public MessageHandler {
 		void saveBans();
 		QVariant getConf(const QString &key, QVariant def);
 		void setConf(const QString &key, const QVariant &value);
-		void dblog(const char *str);
+		void dblog(const QString &str);
 
 		// From msgHandler. Implementation in Messages.cpp
 		virtual void msgSpeex(Connection *, MessageSpeex *);
@@ -298,6 +308,7 @@ class Server : public QThread, public MessageHandler {
 		virtual void msgChannelMove(Connection *, MessageChannelMove *);
 		virtual void msgChannelLink(Connection *, MessageChannelLink *);
 		virtual void msgChannelRename(Connection *, MessageChannelRename *);
+		virtual void msgChannelDescUpdate(Connection *, MessageChannelDescUpdate *);
 		virtual void msgServerBanList(Connection *, MessageServerBanList *);
 		virtual void msgTextMessage(Connection *, MessageTextMessage *);
 		virtual void msgPermissionDenied(Connection *, MessagePermissionDenied *);
@@ -306,6 +317,8 @@ class Server : public QThread, public MessageHandler {
 		virtual void msgTexture(Connection *, MessageTexture *);
 		virtual void msgCryptSetup(Connection *, MessageCryptSetup *);
 		virtual void msgCryptSync(Connection *, MessageCryptSync *);
+		virtual void msgContextAddAction(Connection *, MessageContextAddAction *);
+		virtual void msgContextAction(Connection *, MessageContextAction *);
 };
 
 
