@@ -169,7 +169,7 @@ ServerDB::ServerDB() {
 			SQLDO("CREATE UNIQUE INDEX %1config_key ON %1config(server_id, keystring)");
 			SQLDO("CREATE TRIGGER %1config_server_del AFTER DELETE ON %1servers FOR EACH ROW BEGIN DELETE FROM %1config WHERE server_id = old.server_id; END;");
 
-			SQLDO("CREATE TABLE %1channels (server_id INTEGER NOT NULL, channel_id INTEGER NOT NULL, parent_id INTEGER, name TEXT, inheritacl INTEGER)");
+			SQLDO("CREATE TABLE %1channels (server_id INTEGER NOT NULL, channel_id INTEGER NOT NULL, parent_id INTEGER, name TEXT, inheritacl INTEGER, description TEXT)");
 			SQLDO("CREATE UNIQUE INDEX %1channel_id ON %1channels(server_id, channel_id)");
 			SQLDO("CREATE TRIGGER %1channels_parent_del AFTER DELETE ON %1channels FOR EACH ROW BEGIN DELETE FROM %1channels WHERE parent_id = old.channel_id AND server_id = old.server_id; UPDATE %1players SET lastchannel=0 WHERE lastchannel = old.channel_id AND server_id = old.server_id; END;");
 			SQLDO("CREATE TRIGGER %1channels_server_del AFTER DELETE ON %1servers FOR EACH ROW BEGIN DELETE FROM %1channels WHERE server_id = old.server_id; END;");
@@ -205,7 +205,7 @@ ServerDB::ServerDB() {
 			SQLDO("CREATE TRIGGER %1bans_del_server AFTER DELETE ON %1servers FOR EACH ROW BEGIN DELETE FROM %1bans WHERE server_id = old.server_id; END;");
 
 			SQLDO("INSERT INTO %1servers (server_id) VALUES(1)");
-			SQLDO("INSERT INTO %1meta (keystring, value) VALUES('version','2')");
+			SQLDO("INSERT INTO %1meta (keystring, value) VALUES('version','3')");
 
 			SQLDO("VACUUM");
 		} else {
@@ -220,7 +220,7 @@ ServerDB::ServerDB() {
 			SQLDO("CREATE UNIQUE INDEX %1config_key ON %1config(server_id, keystring)");
 			SQLDO("ALTER TABLE %1config ADD CONSTRAINT %1config_server_del FOREIGN KEY (server_id) REFERENCES %1servers(server_id) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE %1channels (server_id INTEGER NOT NULL, channel_id INTEGER NOT NULL, parent_id INTEGER, name varchar(255), inheritacl INTEGER) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
+			SQLDO("CREATE TABLE %1channels (server_id INTEGER NOT NULL, channel_id INTEGER NOT NULL, parent_id INTEGER, name varchar(255), inheritacl INTEGER, description TEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
 			SQLDO("CREATE UNIQUE INDEX %1channel_id ON %1channels(server_id, channel_id)");
 			SQLDO("ALTER TABLE %1channels ADD CONSTRAINT %1channels_parent_del FOREIGN KEY (server_id, parent_id) REFERENCES %1channels(server_id,channel_id) ON DELETE CASCADE");
 			SQLDO("ALTER TABLE %1channels ADD CONSTRAINT %1channels_server_del FOREIGN KEY (server_id) REFERENCES %1servers(server_id) ON DELETE CASCADE");
@@ -258,7 +258,7 @@ ServerDB::ServerDB() {
 			SQLDO("ALTER TABLE %1bans ADD CONSTRAINT %1bans_del_server FOREIGN KEY(server_id) REFERENCES %1servers(server_id) ON DELETE CASCADE");
 
 			SQLDO("INSERT INTO %1servers (server_id) VALUES(1)");
-			SQLDO("INSERT INTO %1meta (keystring, value) VALUES('version','2')");
+			SQLDO("INSERT INTO %1meta (keystring, value) VALUES('version','3')");
 		}
 
 		if (migrate) {
@@ -288,6 +288,7 @@ ServerDB::ServerDB() {
 
 			SQLDO("UPDATE %1meta SET value='2' WHERE keystring='version'");
 			SQLDO("CREATE UNIQUE INDEX %1players_id ON %1players (server_id, player_id)");
+			SQLDO("ALTER TABLE %1channels ADD COLUMN description TEXT");
 		} else {
 			SQLDO("CREATE TABLE %1slog(server_id INTEGER, msg TEXT, msgtime TIMESTAMP) Type=InnoDB");
 			SQLDO("CREATE INDEX %1slog_time ON %1slog(msgtime)");
@@ -306,6 +307,15 @@ ServerDB::ServerDB() {
 			SQLDO("ALTER TABLE %1players CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci");
 			SQLDO("ALTER TABLE %1servers CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci");
 			SQLDO("ALTER TABLE %1slog CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci");
+			SQLDO("ALTER TABLE %1channels ADD COLUMN description TEXT");
+		}
+	} else if (version < 3) {
+		if (Meta::mp.qsDBDriver == "QSQLITE") {
+			SQLDO("UPDATE %1meta SET value='3' WHERE keystring='version'");
+			SQLDO("ALTER TABLE %1channels ADD COLUMN description TEXT");
+		} else {
+			SQLDO("UPDATE %1meta SET value='3' WHERE keystring='version'");
+			SQLDO("ALTER TABLE %1channels ADD COLUMN description TEXT");
 		}
 	}
 	query.clear();
@@ -458,24 +468,30 @@ int Server::registerPlayer(const QString &name) {
 
 	qhUserIDCache.remove(name);
 
-	int res = dbus->dbusRegisterPlayer(name);
+	int res = -2;
+	emit registerPlayerSig(res, name);
 	if (res != -2) {
 		qhUserNameCache.remove(res);
-		return res;
 	}
+	if (res == -1)
+		return res;
 
 	TransactionHolder th;
 
 	QSqlQuery &query = *th.qsqQuery;
-
-	SQLPREP("SELECT MAX(player_id)+1 AS id FROM %1players WHERE server_id=? AND player_id < 1000000000");
-	query.addBindValue(iServerNum);
-	SQLEXEC();
 	int id = 0;
-	if (query.next())
-		id = query.value(0).toInt();
 
-	SQLPREP("INSERT INTO %1players (server_id, player_id, name) VALUES (?,?,?)");
+	if (res < 0) {
+		SQLPREP("SELECT MAX(player_id)+1 AS id FROM %1players WHERE server_id=? AND player_id < 1000000000");
+		query.addBindValue(iServerNum);
+		SQLEXEC();
+		if (query.next())
+			id = query.value(0).toInt();
+	} else {
+		id = res;
+	}
+
+	SQLPREP("REPLACE INTO %1players (server_id, player_id, name) VALUES (?,?,?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
 	query.addBindValue(name);
@@ -495,9 +511,10 @@ bool Server::unregisterPlayer(int id) {
 	qhUserIDCache.remove(oname);
 	qhUserNameCache.remove(id);
 
-	int res = dbus->dbusUnregisterPlayer(id);
-	if (res >= 0) {
-		return (res > 0);
+	int res = -2;
+	emit unregisterPlayerSig(res, id);
+	if (res == 0) {
+		return false;
 	}
 
 	TransactionHolder th;
@@ -514,7 +531,7 @@ bool Server::unregisterPlayer(int id) {
 QMap<int, QPair<QString, QString> > Server::getRegisteredPlayers(const QString &filter) {
 	QMap<int, QPair<QString, QString> > m;
 
-	m = dbus->dbusGetRegisteredPlayers(filter);
+	emit getRegisteredPlayersSig(filter, m);
 
 	TransactionHolder th;
 
@@ -540,7 +557,8 @@ QMap<int, QPair<QString, QString> > Server::getRegisteredPlayers(const QString &
 
 bool Server::isPlayerId(int id) {
 	QString name, email;
-	int res = dbus->dbusGetRegistration(id, name, email);
+	int res = -2;
+	emit getRegistrationSig(res, id, name, email);
 	if (res >= 0)
 		return (res > 0);
 
@@ -558,7 +576,8 @@ bool Server::isPlayerId(int id) {
 }
 
 bool Server::getRegistration(int id, QString &name, QString &email) {
-	int res = dbus->dbusGetRegistration(id, name, email);
+	int res = -2;
+	emit getRegistrationSig(res, id, name, email);
 	if (res >= 0)
 		return (res > 0);
 
@@ -581,7 +600,10 @@ bool Server::getRegistration(int id, QString &name, QString &email) {
 // -2 Anonymous
 
 int Server::authenticate(QString &name, const QString &pw) {
-	int res = dbus->authenticate(name, pw);
+	int res = -2;
+
+	emit authenticateSig(res, name, pw);
+
 	if (res != -2) {
 		if (res != -1) {
 			TransactionHolder th;
@@ -626,7 +648,9 @@ int Server::authenticate(QString &name, const QString &pw) {
 }
 
 bool Server::setPW(int id, const QString &pw) {
-	int res = dbus->dbusSetPW(id, pw);
+	int res = -2;
+
+	emit setPwSig(res, id, pw);
 	if (res >= 0)
 		return (res > 0);
 
@@ -650,7 +674,9 @@ bool Server::setEmail(int id, const QString &email) {
 	if (id <= 0)
 		return false;
 
-	int res = dbus->dbusSetEmail(id, email);
+	int res = -2;
+	emit setEmailSig(res, id, email);
+
 	if (res >= 0)
 		return (res > 0);
 
@@ -682,7 +708,8 @@ bool Server::setName(int id, const QString &name) {
 	qhUserIDCache.remove(oname);
 	qhUserNameCache.remove(id);
 
-	int res = dbus->dbusSetName(id, name);
+	int res = -2;
+	emit setNameSig(res, id, name);
 	if (res == 0)
 		return false;
 
@@ -703,17 +730,38 @@ bool Server::setTexture(int id, const QByteArray &texture) {
 		return false;
 
 	QByteArray tex;
-	if (texture.size() != 600 * 60 * 4)
-		tex = qUncompress(texture);
-	else
-		tex = texture;
+	if (! texture.isEmpty()) {
+		quint32 l = 600*60*4;
+		if (texture.size() < 4)
+			return false;
 
-	if (tex.size() != 600 * 60 * 4)
-		return false;
+		if (texture.size() != l) {
+			const unsigned char *data = reinterpret_cast<const unsigned char *>(texture.constData());
 
-	tex = qCompress(tex);
+			if (((data[0] << 24) | (data[1] << 16) | (data[2] << 8) | data[3]) == l) {
+				tex = qUncompress(texture);
+			} else {
+				QByteArray qba(4,0);
+				qba[0] = (l >> 24) & 0xFF;
+				qba[1] = (l >> 16) & 0xFF;
+				qba[2] = (l >> 8) & 0xFF;
+				qba[3] = (l >> 0) & 0xFF;
+				qba.append(texture);
+				tex = qUncompress(qba);
+			}
+		} else {
+			tex = texture;
+		}
 
-	int res = dbus->dbusSetTexture(id, tex);
+		if (tex.size() != 600 * 60 * 4)
+			return false;
+	}
+
+	if (! tex.isEmpty())
+		tex = qCompress(tex);
+
+	int res = -2;
+	emit setTextureSig(res, id, tex);
 	if (res >= 0)
 		return (res > 0);
 
@@ -745,7 +793,8 @@ void ServerDB::setSUPW(int srvnum, const QString &pw) {
 }
 
 QString Server::getUserName(int id) {
-	QString name = dbus->mapIdToName(id);
+	QString name;
+	emit idToNameSig(name, id);
 	if (! name.isEmpty())
 		return name;
 
@@ -762,8 +811,8 @@ QString Server::getUserName(int id) {
 }
 
 int Server::getUserID(const QString &name) {
-	int id = dbus->mapNameToId(name);
-
+	int id = -2;
+	emit nameToIdSig(id, name);
 	if (id != -2)
 		return id;
 	TransactionHolder th;
@@ -780,7 +829,8 @@ int Server::getUserID(const QString &name) {
 }
 
 QByteArray Server::getUserTexture(int id) {
-	QByteArray qba=dbus->mapIdToTexture(id);
+	QByteArray qba;
+	emit idToTextureSig(qba, id);
 	if (! qba.isNull()) {
 		return qba;
 	}
@@ -839,7 +889,7 @@ void Server::removeLink(Channel *c, Channel *l) {
 	}
 }
 
-Channel *Server::addChannel(Channel *p, const QString &name) {
+Channel *Server::addChannel(Channel *p, const QString &name, const QString &desc) {
 	TransactionHolder th;
 
 	QSqlQuery &query = *th.qsqQuery;
@@ -852,13 +902,14 @@ Channel *Server::addChannel(Channel *p, const QString &name) {
 		id = query.value(0).toInt();
 
 
-	SQLPREP("INSERT INTO %1channels (server_id, parent_id, channel_id, name) VALUES (?,?,?,?)");
+	SQLPREP("INSERT INTO %1channels (server_id, parent_id, channel_id, name, description) VALUES (?,?,?,?,?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(p->iId);
 	query.addBindValue(id);
 	query.addBindValue(name);
+	query.addBindValue(desc);
 	SQLEXEC();
-	Channel *c = new Channel(id, name, p);
+	Channel *c = new Channel(id, name, desc, p);
 	qhChannels.insert(id, c);
 	return c;
 }
@@ -880,10 +931,11 @@ void Server::updateChannel(const Channel *c) {
 	ChanACL *acl;
 
 	QSqlQuery &query = *th.qsqQuery;
-	SQLPREP("UPDATE %1channels SET name = ?, parent_id = ?, inheritacl = ? WHERE server_id = ? AND channel_id = ?");
+	SQLPREP("UPDATE %1channels SET name = ?, parent_id = ?, inheritacl = ?, description = ? WHERE server_id = ? AND channel_id = ?");
 	query.addBindValue(c->qsName);
 	query.addBindValue(c->cParent ? c->cParent->iId : QVariant());
 	query.addBindValue(c->bInheritACL ? 1 : 0);
+	query.addBindValue(c->qsDesc);
 	query.addBindValue(iServerNum);
 	query.addBindValue(c->iId);
 	SQLEXEC();
@@ -1005,17 +1057,17 @@ void Server::readChannels(Channel *p) {
 	{
 		TransactionHolder th;
 		if (parentid == -1) {
-			SQLPREP("SELECT channel_id, name, inheritacl FROM %1channels WHERE server_id = ? AND parent_id IS NULL ORDER BY name");
+			SQLPREP("SELECT channel_id, name, inheritacl, description FROM %1channels WHERE server_id = ? AND parent_id IS NULL ORDER BY name");
 			query.addBindValue(iServerNum);
 		} else {
-			SQLPREP("SELECT channel_id, name, inheritacl FROM %1channels WHERE server_id = ? AND parent_id=? ORDER BY name");
+			SQLPREP("SELECT channel_id, name, inheritacl, description FROM %1channels WHERE server_id = ? AND parent_id=? ORDER BY name");
 			query.addBindValue(iServerNum);
 			query.addBindValue(parentid);
 		}
 		SQLEXEC();
 
 		while (query.next()) {
-			c = new Channel(query.value(0).toInt(), query.value(1).toString(), p);
+			c = new Channel(query.value(0).toInt(), query.value(1).toString(), query.value(3).toString(), p);
 			if (! p)
 				c->setParent(this);
 			qhChannels.insert(c->iId, c);
@@ -1094,6 +1146,7 @@ void Server::dumpChannel(const Channel *c) {
 	}
 
 	qWarning("Channel %s (ACLInherit %d)", qPrintable(c->qsName), c->bInheritACL);
+	qWarning("Description: %s", qPrintable(c->qsDesc));
 	foreach(g, c->qhGroups) {
 		qWarning("Group %s (Inh %d  Able %d)", qPrintable(g->qsName), g->bInherit, g->bInheritable);
 		foreach(pid, g->qsAdd)
@@ -1184,7 +1237,7 @@ void Server::setConf(const QString &key, const QVariant &value) {
 	ServerDB::setConf(iServerNum, key, value);
 }
 
-void Server::dblog(const char *str) {
+void Server::dblog(const QString &str) {
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
 
