@@ -42,11 +42,6 @@
 
 #ifdef Q_OS_UNIX
 #include "UnixMurmur.h"
-#include <stdio.h>
-#endif
-
-#ifdef Q_OS_WIN
-#include <intrin.h>
 #endif
 
 QFile *qfLog = NULL;
@@ -95,8 +90,7 @@ static void murmurMessageOutput(QtMsgType type, const char *msg) {
 #else
 		qlErrors << m;
 #ifndef QT_NO_DEBUG
-		if (detach)
-			fprintf(stderr, "%s\n", qPrintable(m));
+		fprintf(stderr, "%s\n", qPrintable(m));
 #endif
 #endif
 	} else {
@@ -160,6 +154,7 @@ int main(int argc, char **argv) {
 #else
 	QCoreApplication a(argc, argv);
 	UnixMurmur unixhandler;
+	unixhandler.initialcap();
 #endif
 	a.setApplicationName("Murmur");
 	a.setOrganizationName("Mumble");
@@ -247,26 +242,10 @@ int main(int argc, char **argv) {
 			       "If no inifile is provided, murmur will search for one in \n"
 			       "default locations.", argv[0]);
 #ifdef Q_OS_UNIX
-		} else if (arg == "-descriptors") {
-			QAbstractEventDispatcher *ed = QAbstractEventDispatcher::instance();
-			if (QLatin1String(ed->metaObject()->className()) != QLatin1String("QEventDispatcherGlib"))
-				qWarning("Not running with glib. While you may be able to open more descriptors, sockets above %d will not work", FD_SETSIZE);
-			qWarning("Running descriptor test.");
-			int count;
-			QList<QFile *> ql;
-			for (count=0;count < 524288; ++count) {
-				QFile *qf = new QFile(a.applicationFilePath());
-				if (qf->open(QIODevice::ReadOnly))
-					ql << qf;
-				else
-					break;
-				if ((count & 511) == 0)
-					qWarning("%d descriptors...", count);
-			}
-			foreach(QFile *qf, ql)
-				delete qf;
-			ql.clear();
-			qFatal("Managed to open %d descriptors", count);
+		} else if (arg == "-limits") {
+			unixhandler.setuid();
+			unixhandler.finalcap();
+			LimitTest::testLimits(a);
 #endif
 		} else {
 			detach = false;
@@ -279,6 +258,10 @@ int main(int argc, char **argv) {
 	}
 
 	Meta::mp.read(inifile);
+
+#ifdef Q_OS_UNIX
+	unixhandler.setuid();
+#endif
 	ServerDB db;
 
 	meta = new Meta();
@@ -374,7 +357,10 @@ int main(int argc, char **argv) {
 		dup2(fd, 2);
 		close(fd);
 	}
+	unixhandler.finalcap();
 #endif
+
+#ifdef USE_DBUS
 	MurmurDBus::registerTypes();
 
 	if (! Meta::mp.qsDBus.isEmpty()) {
@@ -398,7 +384,7 @@ int main(int argc, char **argv) {
 			qWarning("Failed to connect to D-Bus %s",qPrintable(Meta::mp.qsDBus));
 		}
 	}
-	meta->dbus = new MetaDBus(meta);
+	new MetaDBus(meta);
 	if (MurmurDBus::qdbc.isConnected()) {
 		if (! MurmurDBus::qdbc.registerObject("/", meta) || ! MurmurDBus::qdbc.registerService(Meta::mp.qsDBusService)) {
 			QDBusError e=MurmurDBus::qdbc.lastError();
@@ -407,6 +393,7 @@ int main(int argc, char **argv) {
 			qWarning("DBus registration succeeded");
 		}
 	}
+#endif
 
 #ifdef USE_ICE
 	IceStart();
